from enum import Enum, auto
import json
import time
from typing import Any, Dict, List, Optional
from unbound.api_resources.apis.logger import Logger
from llama_index.core.callbacks.schema import (
    CBEventType,
)
from uuid import uuid4

try:
    from llama_index.core.callbacks.base_handler import (
        BaseCallbackHandler as LlamaIndexBaseCallbackHandler,
    )
    from llama_index.core.utilities.token_counting import TokenCounter
except ModuleNotFoundError:
    raise ModuleNotFoundError(
        "Please install llama-index to use Unbound Callback Handler"
    )
except ImportError:
    raise ImportError("Please pip install llama-index to use Unbound Callback Handler")


class LlamaIndexCallbackHandler(LlamaIndexBaseCallbackHandler):
    def __init__(
        self,
        api_key: str,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> None:
        super().__init__(
            event_starts_to_ignore=[
                CBEventType.CHUNKING,
                CBEventType.NODE_PARSING,
                CBEventType.SYNTHESIZE,
                CBEventType.EXCEPTION,
                CBEventType.TREE,
                CBEventType.RERANKING,
            ],
            event_ends_to_ignore=[
                CBEventType.CHUNKING,
                CBEventType.NODE_PARSING,
                CBEventType.SYNTHESIZE,
                CBEventType.EXCEPTION,
                CBEventType.TREE,
                CBEventType.RERANKING,
            ],
        )

        self.api_key = api_key
        self.metadata: Dict[str, Any] = metadata or {}
        self.metadata.update({"_source": "LlamaIndex", "_source_type": "Agent"})

        self.unbound_logger = Logger(api_key=api_key)

        self._token_counter = TokenCounter()
        self.completion_tokens = 0
        self.prompt_tokens = 0
        self.token_llm = 0

        self.log_object: Any = []
        self.prompt_records: Any = []

        self.request: Any = {}
        self.response: Any = {}

        self.global_trace_id: str = ""
        self.streamingMode: bool = False

        self.event_map: Any = {}
        self.event_array: List = []
        self.main_span_id: str = ""

        if not api_key:
            raise ValueError("Please provide an API key to use UnboundCallbackHandler")

    def on_event_start(  # type: ignore
        self,
        event_type: Any,
        payload: Optional[Dict[str, Any]] = None,
        event_id: str = "",
        parent_id: str = "",
        **kwargs: Any,
    ) -> str:
        """Run when an event starts and return id of event."""

        span_id = str(event_id)
        parent_span_id = parent_id
        span_name = event_type
        start_time = time.time()

        if parent_id == "root":
            parent_span_id = self.main_span_id

        if event_type == "llm":
            request_payload = self.llm_event_start(payload)
        elif event_type == "embedding":
            request_payload = self.embedding_event_start(payload)
        elif event_type == "agent_step":
            request_payload = self.agent_step_event_start(payload)
        elif event_type == "function_call":
            request_payload = self.function_call_event_start(payload)
        elif event_type == "query":
            request_payload = self.query_event_start(payload)
        elif event_type == "retrieve":
            request_payload = self.retrieve_event_start(payload)
        elif event_type == "templating":
            request_payload = self.templating_event_start(payload)
        elif event_type == "sub_question":
            request_payload = self.sub_question_event_start(payload)
        else:
            return ""

        start_event_information = {
            "span_id": span_id,
            "parent_span_id": parent_span_id,
            "span_name": span_name.value,
            "trace_id": self.global_trace_id,
            "request": request_payload,
            "event_type": event_type,
            "start_time": start_time,
            "metadata": self.metadata,
        }
        self.event_map[span_id] = start_event_information

    def on_event_end(
        self,
        event_type: Any,
        payload: Optional[Dict[str, Any]] = None,
        event_id: str = "",
        **kwargs: Any,
    ) -> None:
        """Run when an event ends."""
        span_id = event_id

        if payload is None:
            response_payload = {}
            if span_id in self.event_map:
                event = self.event_map[event_id]
                start_time = event["start_time"]
                end_time = time.time()
                total_time = f"{((end_time - start_time) * 1000):04.0f}"
                response_payload["response_time"] = total_time
        else:
            if event_type == "llm":
                response_payload = self.llm_event_end(payload, event_id)
            elif event_type == "embedding":
                response_payload = self.embedding_event_end(payload, event_id)
            elif event_type == "agent_step":
                response_payload = self.agent_step_event_end(payload, event_id)
            elif event_type == "function_call":
                response_payload = self.function_call_event_end(payload, event_id)
            elif event_type == "query":
                response_payload = self.query_event_end(payload, event_id)
            elif event_type == "retrieve":
                response_payload = self.retrieve_event_end(payload, event_id)
            elif event_type == "templating":
                response_payload = self.templating_event_end(payload, event_id)
            elif event_type == "sub_question":
                response_payload = self.sub_question_event_end(payload, event_id)
            else:
                return

        self.event_map[span_id]["response"] = response_payload

        self.event_array.append(self.event_map[span_id])

    def start_trace(self, trace_id: Optional[str] = None) -> None:
        """Run when an overall trace is launched."""
        if trace_id == "index_construction":
            self.global_trace_id = self.metadata.get("traceId", str(uuid4()))  # type: ignore [union-attr]

        self.main_span_id = ""

    def end_trace(
        self,
        trace_id: Optional[str] = None,
        trace_map: Optional[Dict[str, List[str]]] = None,
    ) -> None:
        """Run when an overall trace is exited."""

        self.log_object = self.event_array

        self.unbound_logger.log(log_object=self.log_object)
        self.event_array = []

    # ------------------- EVENT Handlers ------------------- #
    def llm_event_start(self, payload: Any) -> Any:
        self.request = {}
        if "messages" in payload:
            chunks = payload.get("messages", {})
            self.prompt_tokens = self._token_counter.estimate_tokens_in_messages(chunks)
            messages = payload.get("messages", {})
            self.prompt_records = [
                {"role": m.role.value, "content": m.content} for m in messages
            ]
        self.request["method"] = "POST"
        self.request["url"] = payload.get("serialized", {}).get(
            "api_base", "chat/completions"
        )
        self.request["provider"] = payload.get("serialized", {}).get("class_name", "")
        self.request["headers"] = {}
        self.request["body"] = {"messages": self.prompt_records}
        self.request["body"].update(
            {"model": payload.get("serialized", {}).get("model", "")}
        )
        self.request["body"].update(
            {"temperature": payload.get("serialized", {}).get("temperature", "")}
        )

        return self.request

    def llm_event_end(self, payload: Any, event_id) -> Any:
        result: Dict[str, Any] = {}
        result["body"] = {}

        try:
            data = self.serialize(payload)
        except Exception:
            data = payload.__dict__

        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]
        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        chunks = payload.get("messages", {})
        self.completion_tokens = self._token_counter.estimate_tokens_in_messages(chunks)
        self.token_llm = self.prompt_tokens + self.completion_tokens

        result["body"] = data["response"]
        result["body"].update(
            {
                "usage": {
                    "prompt_tokens": self.prompt_tokens,
                    "completion_tokens": self.completion_tokens,
                    "total_tokens": self.token_llm,
                }
            }
        )
        result["body"].update({"id": event_id})
        result["body"].update({"created": int(time.time())})
        result["body"].update({"model": getattr(data, "model", "")})
        result["streamingMode"] = self.streamingMode

        result["status"] = 200
        result["headers"] = {}
        result["response_time"] = total_time

        return result

    # ------------------------------------------------------ #
    def embedding_event_start(self, payload: Any) -> Any:
        self.request = {}
        if "serialized" in payload:
            self.request["method"] = "POST"
            self.request["url"] = payload.get("serialized", {}).get(
                "api_base", "embeddings"
            )
            self.request["provider"] = payload.get("serialized", {}).get(
                "class_name", ""
            )
            self.request["headers"] = {}
            self.request["body"] = {
                "model": payload.get("serialized", {}).get("model_name", "")
            }

        return self.request

    def embedding_event_end(self, payload: Any, event_id) -> Any:
        if event_id in self.event_map:
            event = self.event_map[event_id]
            event["request"]["body"]["input"] = payload.get("chunks", "")
            # Setting as ...INPUT... to avoid logging the entire data input file
            # event["request"]["body"]["input"] = "...INPUT..."

            start_time = event["start_time"]

        self.response = {}

        chunk_str = str(payload.get("chunks", ""))
        embd_str = str(payload.get("embeddings", ""))

        self.prompt_tokens = self._token_counter.get_string_tokens(chunk_str)
        self.completion_tokens = self._token_counter.get_string_tokens(embd_str)

        self.response["status"] = 200
        self.response["body"] = {
            "embedding": "...REDACTED...",
        }

        self.response["body"].update({"created": int(time.time())})

        self.response["body"].update(
            {
                "usage": {
                    "prompt_tokens": self.prompt_tokens,
                    "completion_tokens": self.completion_tokens,
                    "total_tokens": self.prompt_tokens + self.completion_tokens,
                }
            }
        )
        self.response["headers"] = {}

        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        self.response["response_time"] = total_time

        return self.response

    # ------------------------------------------------------ #
    def agent_step_event_start(self, payload: Any) -> Any:
        try:
            data = json.dumps(self.serialize(payload))
        except Exception:
            data = json.dumps(payload.__dict__)
        return data

    def agent_step_event_end(self, payload: Any, event_id) -> Any:
        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]
        data = self.serialize(payload)
        result = self.transform_agent_step_end(data)
        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        result["response_time"] = total_time
        return result

    # ------------------------------------------------------ #
    def function_call_event_start(self, payload: Any) -> Any:
        result = self.transform_function_call_start(payload)
        return result

    def function_call_event_end(self, payload: Any, event_id) -> Any:
        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]
        data = self.serialize(payload)
        result = self.transform_function_call_end(data)
        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        result["response_time"] = total_time
        return result

    # ------------------------------------------------------ #
    def query_event_start(self, payload: Any) -> Any:
        try:
            data = json.dumps(self.serialize(payload))
        except Exception:
            data = json.dumps(payload.__dict__)
        return data

    def query_event_end(self, payload: Any, event_id) -> Any:
        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]
        data = self.serialize(payload)
        result = self.transform_query_end(data)
        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        result["response_time"] = total_time
        return result

    # ------------------------------------------------------ #
    def retrieve_event_start(self, payload: Any) -> Any:
        try:
            data = json.dumps(self.serialize(payload))
        except Exception:
            data = json.dumps(payload.__dict__)
        return data

    def retrieve_event_end(self, payload: Any, event_id) -> Any:
        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]

        data = self.serialize(payload)
        result = self.transform_retrieve_end(data)
        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        result["response_time"] = total_time
        return result

    # ------------------------------------------------------ #
    def templating_event_start(self, payload: Any) -> Any:
        data = self.serialize(payload)
        result = self.transform_templating_start(data)
        return result

    def templating_event_end(self, payload: Any, event_id) -> Any:
        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]
        result = self.transform_templating_end(event_id)

        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        result["response_time"] = total_time
        return result

    # ------------------------------------------------------ #

    def sub_question_event_start(self, payload: Any) -> Any:
        try:
            data = json.dumps(self.serialize(payload))
        except Exception:
            data = json.dumps(payload.__dict__)

        return data

    def sub_question_event_end(self, payload: Any, event_id) -> Any:
        result: Dict[str, Any] = {}
        result["body"] = {}
        if event_id in self.event_map:
            event = self.event_map[event_id]
            start_time = event["start_time"]

        try:
            data = self.serialize(payload)
        except Exception:
            data = payload.__dict__

        end_time = time.time()
        total_time = f"{((end_time - start_time) * 1000):04.0f}"

        result["body"] = data
        result["body"]["response_time"] = total_time

        return result

    # ------------------------------------------------------ #

    # ----------------- EVENT Transformers ----------------- #
    def transform_agent_step_end(self, data: Any) -> Any:
        try:
            output_data = {
                "agent_chat_response": {
                    "response": data["response"]["response"],
                    "sources": [
                        {
                            "content": source["content"],
                            "tool_name": source["tool_name"],
                            "raw_input": source["raw_input"],
                            "raw_output": {
                                "response": source["raw_output"]["response"],
                                "source_nodes": [
                                    {
                                        "id": node["node"]["id_"],
                                        "metadata": node["node"]["metadata"],
                                        "text_excerpt": node["node"]["text"],
                                        "score": node["score"],
                                    }
                                    for node in source["raw_output"]["source_nodes"]
                                ],
                            },
                        }
                        for source in data["response"]["sources"]
                    ],
                }
            }
            return output_data
        except Exception:
            return data

    def transform_query_end(self, data: Any) -> Any:
        try:
            output_data = {
                "response": {
                    "content": data["response"]["response"],
                    "source_nodes": [],
                }
            }

            for source_node in data["response"]["source_nodes"]:
                node = source_node["node"]
                metadata = node["metadata"]
                text_excerpt = node["text"][
                    node["start_char_idx"] : node["end_char_idx"]
                ]

                output_data["response"]["source_nodes"].append(
                    {
                        "id": node["id_"],
                        "metadata": {
                            "file_path": metadata["file_path"],
                            "file_name": metadata["file_name"],
                            "file_type": metadata["file_type"],
                            "file_size": metadata["file_size"],
                            "creation_date": metadata["creation_date"],
                            "last_modified_date": metadata["last_modified_date"],
                        },
                        "text_excerpt": text_excerpt,
                        "score": source_node["score"],
                    }
                )
            return output_data
        except Exception:
            return data

    def transform_retrieve_end(self, data: Any) -> Any:
        try:
            output_data = {"nodes": []}  # type: ignore[var-annotated]

            for node_data in data["nodes"]:
                node = node_data["node"]
                metadata = node["metadata"]
                text_excerpt = node["text"][
                    node["start_char_idx"] : node["end_char_idx"]
                ]

                relationships = {}
                for relationship, details in node["relationships"].items():
                    relationships[relationship.name] = {
                        "node_id": details["node_id"],
                        "node_type": "DOCUMENT"
                        if relationship == NodeRelationship.SOURCE
                        else "TEXT",
                    }

                output_data["nodes"].append(
                    {
                        "id": node["id_"],
                        "metadata": {
                            "file_path": metadata["file_path"],
                            "file_name": metadata["file_name"],
                            "file_type": metadata["file_type"],
                            "file_size": metadata["file_size"],
                            "creation_date": metadata["creation_date"],
                            "last_modified_date": metadata["last_modified_date"],
                        },
                        "relationships": relationships,
                        "text_excerpt": text_excerpt,
                        "score": node_data["score"],
                    }
                )

            return output_data
        except Exception:
            return data

    def transform_templating_start(self, data: Any) -> Any:
        try:
            output_data = {
                "template": {
                    "system": data["template"].split("user:")[0].strip(),
                    "user": "Context information is below.\n---------------------\n{context_str}\n---------------------\nGiven the context information and not prior knowledge, answer the query.\nQuery: {query_str}\nAnswer: ",  # noqa: E501
                    "assistant": "",
                },
                "template_vars": {
                    "context_str": data["template_vars"]["context_str"],
                    "query_str": data["template_vars"]["query_str"],
                },
                "system_prompt": data["system_prompt"],
                "query_wrapper_prompt": data["query_wrapper_prompt"],
            }
            return output_data
        except Exception:
            return data

    def transform_templating_end(self, event_id) -> Any:
        try:
            request_data = self.event_map[event_id]["request"]
            context_str = request_data["template_vars"]["context_str"]
            query_str = request_data["template_vars"]["query_str"]
            replace_str = request_data["template"]["user"]
            output_data = {
                "template": {
                    "system": request_data["template"]["system"],
                    "user": replace_str.format(
                        context_str=context_str, query_str=query_str
                    ),
                    "assistant": request_data["template"]["assistant"],
                }
            }

            return output_data
        except Exception:
            return ""

    def transform_function_call_start(self, data: Any) -> Any:
        try:
            tool_meta = data.get("tool")
            output_data = {
                "function_call": data.get("function_call", ""),
                "tool": {
                    "description": tool_meta.description
                    if tool_meta.description
                    else "",
                    "name": tool_meta.name if tool_meta.name else "",
                },
            }
            return output_data
        except Exception:
            return data

    def transform_function_call_end(self, data: Any) -> Any:
        try:
            output_data = {"function_call_response": data["function_call_response"]}
            return output_data
        except Exception:
            return data

    # ----------------- HELPER FUNCTIONS ------------------- #

    def serialize(self, obj):
        if isinstance(obj, Enum):
            return obj.value
        if hasattr(obj, "__dict__"):
            return {key: self.serialize(value) for key, value in obj.__dict__.items()}
        if isinstance(obj, list):
            return [self.serialize(item) for item in obj]
        if isinstance(obj, dict):
            return {key: self.serialize(value) for key, value in obj.items()}
        if isinstance(obj, tuple):
            return tuple(self.serialize(item) for item in obj)
        return obj


class NodeRelationship(str, Enum):
    SOURCE = auto()
    PREVIOUS = auto()
    NEXT = auto()
    PARENT = auto()
    CHILD = auto()



import json
from typing import Union
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.client import AsyncUnbound, Unbound
from unbound.api_resources.types.models_type import Model, ModelDeleted, ModelList
from ..._vendor.openai._types import NotGiven, NOT_GIVEN


class Models(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def list(self, **kwargs) -> ModelList:
        response = self.openai_client.with_raw_response.models.list(**kwargs)
        data = ModelList(**json.loads(response.text))
        data._headers = response.headers
        return data

    def retrieve(
        self, model: str, *, timeout: Union[float, NotGiven] = NOT_GIVEN, **kwargs
    ) -> Model:
        response = self.openai_client.with_raw_response.models.retrieve(
            model=model, timeout=timeout, extra_body=kwargs
        )
        data = Model(**json.loads(response.text))
        data._headers = response.headers
        return data

    def delete(
        self, model: str, *, timeout: Union[float, NotGiven] = NOT_GIVEN, **kwargs
    ) -> ModelDeleted:
        response = self.openai_client.with_raw_response.models.delete(
            model=model, timeout=timeout, extra_body=kwargs
        )
        data = ModelDeleted(**json.loads(response.text))
        data._headers = response.headers
        return data


class AsyncModels(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def list(self, **kwargs) -> ModelList:
        response = await self.openai_client.with_raw_response.models.list(**kwargs)
        data = ModelList(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def retrieve(
        self, model: str, *, timeout: Union[float, NotGiven] = NOT_GIVEN, **kwargs
    ) -> Model:
        response = await self.openai_client.with_raw_response.models.retrieve(
            model=model, timeout=timeout, extra_body=kwargs
        )
        data = Model(**json.loads(response.text))
        data._headers = response.headers
        return data

    async def delete(
        self, model: str, *, timeout: Union[float, NotGiven] = NOT_GIVEN, **kwargs
    ) -> ModelDeleted:
        response = await self.openai_client.with_raw_response.models.delete(
            model=model, timeout=timeout, extra_body=kwargs
        )
        data = ModelDeleted(**json.loads(response.text))
        data._headers = response.headers
        return data



import json
from typing import Any, Dict, List, Optional, Union
import httpx
from pydantic import BaseModel, PrivateAttr
from unbound.api_resources.types.utils import parse_headers


class ConfigAddResponse(BaseModel):
    id: Optional[str]
    version_id: Optional[str]
    slug: Optional[str]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ConfigGetResponse(BaseModel):
    id: Optional[str]
    organisation_id: Optional[str] = None
    config: Optional[Union[str, dict]]
    name: Optional[str]
    workspace_id: Optional[str]
    slug: Optional[str]
    is_default: Optional[int]
    status: Optional[str]
    owner_id: Optional[str]
    created_at: Optional[str]
    updated_by: Optional[str]
    last_updated_at: Optional[str]
    format: Optional[str]
    type: Optional[str]
    version_id: Optional[str]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ConfigListResponse(BaseModel):
    object: Optional[str]
    total: Optional[int]
    data: Optional[List[Dict[str, Any]]]

    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class ConfigUpdateResponse(BaseModel):
    version_id: Optional[str]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import override

from . import resources, _load_client
from ._utils import LazyProxy


class ChatProxy(LazyProxy[resources.Chat]):
    @override
    def __load__(self) -> resources.Chat:
        return _load_client().chat


class BetaProxy(LazyProxy[resources.Beta]):
    @override
    def __load__(self) -> resources.Beta:
        return _load_client().beta


class FilesProxy(LazyProxy[resources.Files]):
    @override
    def __load__(self) -> resources.Files:
        return _load_client().files


class AudioProxy(LazyProxy[resources.Audio]):
    @override
    def __load__(self) -> resources.Audio:
        return _load_client().audio


class ImagesProxy(LazyProxy[resources.Images]):
    @override
    def __load__(self) -> resources.Images:
        return _load_client().images


class ModelsProxy(LazyProxy[resources.Models]):
    @override
    def __load__(self) -> resources.Models:
        return _load_client().models


class BatchesProxy(LazyProxy[resources.Batches]):
    @override
    def __load__(self) -> resources.Batches:
        return _load_client().batches


class EmbeddingsProxy(LazyProxy[resources.Embeddings]):
    @override
    def __load__(self) -> resources.Embeddings:
        return _load_client().embeddings


class CompletionsProxy(LazyProxy[resources.Completions]):
    @override
    def __load__(self) -> resources.Completions:
        return _load_client().completions


class ModerationsProxy(LazyProxy[resources.Moderations]):
    @override
    def __load__(self) -> resources.Moderations:
        return _load_client().moderations


class FineTuningProxy(LazyProxy[resources.FineTuning]):
    @override
    def __load__(self) -> resources.FineTuning:
        return _load_client().fine_tuning


chat: resources.Chat = ChatProxy().__as_proxied__()
beta: resources.Beta = BetaProxy().__as_proxied__()
files: resources.Files = FilesProxy().__as_proxied__()
audio: resources.Audio = AudioProxy().__as_proxied__()
images: resources.Images = ImagesProxy().__as_proxied__()
models: resources.Models = ModelsProxy().__as_proxied__()
batches: resources.Batches = BatchesProxy().__as_proxied__()
embeddings: resources.Embeddings = EmbeddingsProxy().__as_proxied__()
completions: resources.Completions = CompletionsProxy().__as_proxied__()
moderations: resources.Moderations = ModerationsProxy().__as_proxied__()
fine_tuning: resources.FineTuning = FineTuningProxy().__as_proxied__()



import json
from typing import Any, Dict, List, Optional
import httpx
from pydantic import BaseModel, PrivateAttr
from unbound.api_resources.types.utils import parse_headers


class LogsExportCreateResponse(BaseModel):
    id: Optional[str]
    total: Optional[int]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportListResponse(BaseModel):
    object: Optional[str]
    total: Optional[int]
    data: Optional[List[Dict[str, Any]]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportUpdateResponse(BaseModel):
    id: Optional[str]
    total: Optional[int]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportCancelResponse(BaseModel):
    message: Optional[str]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportRetrieveResponse(BaseModel):
    id: Optional[str]
    organisation_id: Optional[str]
    filters: Optional[Dict[str, Any]]
    requested_data: Optional[List[str]]
    status: Optional[str]
    description: Optional[str]
    created_at: Optional[str]
    last_updated_at: Optional[str]
    created_by: Optional[str]
    workspace_id: Optional[str]
    total_records: Optional[int]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportStartResponse(BaseModel):
    message: Optional[str]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class LogsExportDownloadResponse(BaseModel):
    signed_url: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



import json
from typing import Dict, Optional
import httpx
from .utils import parse_headers
from typing import List, Any
from pydantic import BaseModel, PrivateAttr

__all__ = ["CompletionUsage", "Logprobs", "CompletionChoice", "TextCompletion"]


class CompletionUsage(BaseModel):
    completion_tokens: Optional[int]
    prompt_tokens: Optional[int]
    total_tokens: Optional[int]


class Logprobs(BaseModel):
    text_offset: Optional[List[int]] = None
    token_logprobs: Optional[List[Optional[float]]] = None
    tokens: Optional[List[str]] = None
    top_logprobs: Optional[List[Optional[Dict[str, float]]]] = None


class CompletionChoice(BaseModel):
    finish_reason: Optional[str]
    index: Optional[int]
    logprobs: Optional[Logprobs] = None
    text: Optional[str]


class TextCompletion(BaseModel):
    id: Optional[str]
    choices: List[CompletionChoice]
    created: Optional[int]
    model: Optional[str]
    object: Optional[str]
    system_fingerprint: Optional[str] = None
    usage: Optional[CompletionUsage] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class TextChoice(BaseModel, extra="allow"):
    index: Optional[int] = None
    text: Optional[str] = None
    logprobs: Optional[Logprobs] = None
    finish_reason: Optional[str] = None

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class TextCompletionChunk(BaseModel, extra="allow"):
    id: Optional[str] = None
    object: Optional[str] = None
    created: Optional[int] = None
    model: Optional[str] = None
    provider: Optional[str] = None
    choices: List[TextChoice]

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



from unbound.api_resources.base_client import APIClient, AsyncAPIClient
import asyncio


class APIResource:
    _client: APIClient
    # _get: Any
    # _patch: Any
    # _put: Any
    # _delete: Any

    def __init__(self, client: APIClient) -> None:
        self._client = client
        # self._get = client.get
        # self._patch = client.patch
        # self._put = client.put
        # self._delete = client.delete

    def _post(self, *args, **kwargs):
        return self._client._post(*args, **kwargs)

    def _put(self, *args, **kwargs):
        return self._client._put(*args, **kwargs)

    def _get(self, *args, **kwargs):
        return self._client._get(*args, **kwargs)

    def _delete(self, *args, **kwargs):
        return self._client._delete(*args, **kwargs)


class AsyncAPIResource:
    _client: AsyncAPIClient

    def __init__(self, client: AsyncAPIClient) -> None:
        self._client = client
        # self._get = client.get
        # self._patch = client.patch
        # self._put = client.put
        # self._delete = client.delete

    async def _post(self, *args, **kwargs):
        return await self._client._post(*args, **kwargs)

    async def _put(self, *args, **kwargs):
        return await self._client._put(*args, **kwargs)

    async def _get(self, *args, **kwargs):
        return await self._client._get(*args, **kwargs)

    async def _delete(self, *args, **kwargs):
        return await self._client._delete(*args, **kwargs)

    async def _sleep(self, seconds: float) -> None:
        await asyncio.sleep(seconds)



import json
from typing import Any, Dict, List, Optional
import httpx
from pydantic import BaseModel, PrivateAttr
from unbound.api_resources.types.utils import parse_headers


class UserRetrieveResponse(BaseModel):
    object: Optional[str]
    id: Optional[str]
    first_name: Optional[str]
    last_name: Optional[str]
    role: Optional[str]
    email: Optional[str]
    created_at: Optional[str]
    last_updated_at: Optional[str]
    workspace_ids: Optional[List[str]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class UserRetrieveAllResponse(BaseModel):
    total: Optional[int]
    object: Optional[str]
    data: Optional[List[UserRetrieveResponse]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class UserInviteResponse(BaseModel):
    id: Optional[str]
    invite_link: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class UserInviteRetrieveResponse(BaseModel):
    object: Optional[str]
    id: Optional[str]
    email: Optional[str]
    role: Optional[str]
    created_at: Optional[str]
    expires_at: Optional[str]
    accepted_at: Optional[str]
    status: Optional[str]
    invited_by: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class UserInviteRetrieveAllResponse(BaseModel):
    object: Optional[str]
    total: Optional[int]
    data: Optional[List[UserInviteRetrieveResponse]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class WorkspacesAddResponse(BaseModel):
    id: Optional[str]
    slug: Optional[str]
    name: Optional[str]
    description: Optional[str]
    created_at: Optional[str]
    last_updated_at: Optional[str]
    defaults: Optional[Dict[str, Any]]
    users: Optional[List[Dict[str, str]]]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class WorkspacesGetResponse(BaseModel):
    object: Optional[str]
    id: Optional[str]
    slug: Optional[str]
    name: Optional[str]
    description: Optional[str]
    created_at: Optional[str]
    last_updated_at: Optional[str]
    defaults: Optional[Dict[str, Any]]
    is_default: Optional[int] = None
    users: Optional[List[Dict[str, Any]]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class WorkspacesListResponse(BaseModel):
    total: Optional[int]
    object: Optional[str]
    data: Optional[List[WorkspacesGetResponse]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class WorkspacesUpdateResponse(BaseModel):
    id: Optional[str]
    slug: Optional[str]
    name: Optional[str]
    description: Optional[str]
    created_at: Optional[str]
    is_default: Optional[int]
    last_updated_at: Optional[str]
    defaults: Optional[Dict[str, Any]]
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class WorkspaceMemberGetResponse(BaseModel):
    first_name: Optional[str]
    last_name: Optional[str]
    org_role: Optional[str]
    role: Optional[str]
    created_at: Optional[str]
    last_updated_at: Optional[str]
    status: Optional[str]
    workspace_id: Optional[str]
    scopes: Optional[List[str]] = None
    settings: Optional[Any] = None
    object: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default


class WorkspaceMemberListResponse(BaseModel):
    total: Optional[int]
    object: Optional[str]
    data: Optional[List[WorkspaceMemberGetResponse]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)

    def __str__(self):
        return json.dumps(self.dict(), indent=4, default=str)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



from __future__ import annotations
import warnings
from typing import Literal, Optional, Union, Mapping, Any, overload
from unbound.api_resources.base_client import APIClient, AsyncAPIClient
from unbound.api_resources.types.generation_type import (
    PromptCompletion,
    PromptCompletionChunk,
    PromptRender,
)
from unbound.api_resources.utils import (
    retrieve_config,
    GenericResponse,
)

from unbound.api_resources.streaming import AsyncStream, Stream
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource


class Generations(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    def create(
        self,
        *,
        prompt_id: str,
        config: Optional[Union[Mapping, str]] = None,
        variables: Optional[Mapping[str, Any]] = None,
    ) -> Union[GenericResponse, Stream[GenericResponse]]:
        warning_message = "This API has been deprecated. Please use the Prompt API for the saved prompt."  # noqa: E501
        warnings.warn(
            warning_message,
            DeprecationWarning,
            stacklevel=2,
        )
        if config is None:
            config = retrieve_config()
        body = {"variables": variables}
        response = self._post(
            f"/v1/prompts/{prompt_id}/generate",
            body=body,
            mode=None,
            params=None,
            cast_to=GenericResponse,
            stream_cls=Stream[GenericResponse],
            stream=False,
        )
        response["warning"] = warning_message
        return response


class AsyncGenerations(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    async def create(
        self,
        *,
        prompt_id: str,
        config: Optional[Union[Mapping, str]] = None,
        variables: Optional[Mapping[str, Any]] = None,
    ) -> Union[GenericResponse, AsyncStream[GenericResponse]]:
        warning_message = "This API has been deprecated. Please use the Prompt API for the saved prompt."  # noqa: E501
        warnings.warn(
            warning_message,
            DeprecationWarning,
            stacklevel=2,
        )
        if config is None:
            config = retrieve_config()
        body = {"variables": variables}
        response = await self._post(
            f"/v1/prompts/{prompt_id}/generate",
            body=body,
            mode=None,
            params=None,
            cast_to=GenericResponse,
            stream_cls=AsyncStream[GenericResponse],
            stream=False,
        )
        response["warning"] = warning_message
        return response


class Prompts(APIResource):
    completions: Completions

    def __init__(self, client: APIClient) -> None:
        super().__init__(client)
        self.completions = Completions(client)

    def render(
        self,
        *,
        prompt_id: str,
        variables: Mapping[str, Any],
        stream: bool = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> PromptRender:
        """Prompt render Method"""
        body = {
            "variables": variables,
            "temperature": temperature,
            "max_tokens": max_tokens,
            "top_k": top_k,
            "top_p": top_p,
            "stream": stream,
            **kwargs,
        }
        return self._post(
            f"/prompts/{prompt_id}/render",
            body=body,
            params=None,
            cast_to=PromptRender,
            stream_cls=Stream[PromptRender],
            stream=False,
            headers={},
        )


class AsyncPrompts(AsyncAPIResource):
    completions: AsyncCompletions

    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)
        self.completions = AsyncCompletions(client)

    async def render(
        self,
        *,
        prompt_id: str,
        variables: Mapping[str, Any],
        stream: bool = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> PromptRender:
        """Prompt render Method"""
        body = {
            "variables": variables,
            "temperature": temperature,
            "max_tokens": max_tokens,
            "top_k": top_k,
            "top_p": top_p,
            "stream": stream,
            **kwargs,
        }
        return await self._post(
            f"/prompts/{prompt_id}/render",
            body=body,
            params=None,
            cast_to=PromptRender,
            stream=False,
            stream_cls=AsyncStream[PromptRender],
            headers={},
        )


class Completions(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    @overload
    def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: Literal[True],
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> Stream[PromptCompletionChunk]:
        ...

    @overload
    def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: Literal[False] = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> PromptCompletion:
        ...

    @overload
    def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: bool = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> Union[PromptCompletion, Stream[PromptCompletionChunk]]:
        ...

    def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: bool = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> Union[PromptCompletion, Stream[PromptCompletionChunk],]:
        """Prompt completions Method"""
        if config is None:
            config = retrieve_config()
        body = {
            "variables": variables,
            "temperature": temperature,
            "max_tokens": max_tokens,
            "top_k": top_k,
            "top_p": top_p,
            "stream": stream,
            **kwargs,
        }
        return self._post(
            f"/prompts/{prompt_id}/completions",
            body=body,
            params=None,
            cast_to=PromptCompletion,
            stream_cls=Stream[PromptCompletionChunk],
            stream=stream,
            headers={},
        )


class AsyncCompletions(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    @overload
    async def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: Literal[True],
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> AsyncStream[PromptCompletionChunk]:
        ...

    @overload
    async def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: Literal[False] = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> PromptCompletion:
        ...

    @overload
    async def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: bool = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> Union[PromptCompletion, AsyncStream[PromptCompletionChunk]]:
        ...

    async def create(
        self,
        *,
        prompt_id: str,
        variables: Optional[Mapping[str, Any]] = None,
        config: Optional[Union[Mapping, str]] = None,
        stream: bool = False,
        temperature: Optional[float] = None,
        max_tokens: Optional[int] = None,
        top_k: Optional[int] = None,
        top_p: Optional[float] = None,
        **kwargs,
    ) -> Union[PromptCompletion, AsyncStream[PromptCompletionChunk]]:
        """Prompt completions Method"""
        if config is None:
            config = retrieve_config()
        body = {
            "variables": variables,
            "temperature": temperature,
            "max_tokens": max_tokens,
            "top_k": top_k,
            "top_p": top_p,
            "stream": stream,
            **kwargs,
        }
        return await self._post(
            f"/prompts/{prompt_id}/completions",
            body=body,
            params=None,
            cast_to=PromptCompletion,
            stream_cls=AsyncStream[PromptCompletionChunk],
            stream=stream,
            headers={},
        )



import json
from typing import Dict, Optional, Union
import httpx
from .utils import parse_headers
from typing import List, Any
from pydantic import BaseModel, PrivateAttr

__all__ = [
    "Assistant",
    "AssistantList",
    "AssistantDeleted",
    "ToolCodeInterpreter",
    "ToolRetrieval",
    "ToolFunction",
    "Tool",
]


class ToolCodeInterpreter(BaseModel):
    type: Optional[str]


class ToolRetrieval(BaseModel):
    type: Optional[str]


class ToolFunction(BaseModel):
    type: Optional[str]


Tool = Union[ToolCodeInterpreter, ToolRetrieval, ToolFunction]


class Assistant(BaseModel):
    id: Optional[str]
    created_at: Optional[int]
    description: Optional[str] = None
    file_ids: Optional[List[str]] = None
    instructions: Optional[str] = None
    metadata: Optional[object] = None
    model: Optional[str]
    name: Optional[str] = None
    object: Optional[str]
    tools: Optional[List[Tool]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class AssistantList(BaseModel, extra="allow"):
    object: Optional[str]
    data: Optional[List[Assistant]]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class AssistantDeleted(BaseModel, extra="allow"):
    id: Optional[str]
    object: Optional[str]
    deleted: Optional[bool]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


# class AssistantFile(BaseModel, extra="allow"):
#     id: Optional[str]
#     assistant_id: Optional[str]
#     created_at: Optional[int]
#     object: Optional[str]
#     _headers: Optional[httpx.Headers] = PrivateAttr()

#     def __str__(self):
#         del self._headers
#         return json.dumps(self.dict(), indent=4)

#     def get_headers(self) -> Optional[Dict[str, str]]:
#         return parse_headers(self._headers)


# class AssistantFileList(BaseModel, extra="allow"):
#     object: Optional[str]
#     data: Optional[List[AssistantFile]]
#     first_id: Optional[str]
#     last_id: Optional[str]
#     has_more: Optional[bool]
#     _headers: Optional[httpx.Headers] = PrivateAttr()

#     def __str__(self):
#         del self._headers
#         return json.dumps(self.dict(), indent=4)

#     def get_headers(self) -> Optional[Dict[str, str]]:
#         return parse_headers(self._headers)


# class AssistantFileDeleted(BaseModel, extra="allow"):
#     id: Optional[str]
#     deleted: Optional[bool]
#     object: Optional[str]
#     _headers: Optional[httpx.Headers] = PrivateAttr()

#     def __str__(self):
#         del self._headers
#         return json.dumps(self.dict(), indent=4)

#     def get_headers(self) -> Optional[Dict[str, str]]:
#         return parse_headers(self._headers)



import json
from typing import Any, List, Union
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from ..._vendor.openai._types import NotGiven, NOT_GIVEN, FileTypes
from unbound.api_resources.client import AsyncUnbound, Unbound
import typing

from unbound.api_resources.types.audio_types import Transcription, Translation


class Audio(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.transcriptions = Transcriptions(client)
        self.translations = Translations(client)
        self.speech = Speech(client)


class Transcriptions(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    @typing.no_type_check
    def create(
        self,
        *,
        file: FileTypes,
        model: str,
        language: Union[str, NotGiven] = NOT_GIVEN,
        prompt: Union[str, NotGiven] = NOT_GIVEN,
        response_format: Union[str, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        timestamp_granularities: Union[List[str], NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> Transcription:
        extra_headers = kwargs.pop("extra_headers", {})
        response = self.openai_client.with_raw_response.audio.transcriptions.create(
            file=file,
            model=model,
            language=language,
            prompt=prompt,
            response_format=response_format,
            temperature=temperature,
            timestamp_granularities=timestamp_granularities,
            extra_headers=extra_headers,
            extra_body=kwargs,
        )
        data = Transcription(**json.loads(response.text))
        data._headers = response.headers

        return data


class Translations(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(
        self,
        *,
        file: FileTypes,
        model: str,
        prompt: Union[str, NotGiven] = NOT_GIVEN,
        response_format: Union[str, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> Translation:
        extra_headers = kwargs.pop("extra_headers", {})
        response = self.openai_client.with_raw_response.audio.translations.create(
            file=file,
            model=model,
            prompt=prompt,
            response_format=response_format,
            temperature=temperature,
            extra_headers=extra_headers,
            extra_body=kwargs,
        )
        data = Translation(**json.loads(response.text))
        data._headers = response.headers

        return data


class Speech(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    @typing.no_type_check
    def create(
        self,
        *,
        input: str,
        model: str,
        voice: str,
        response_format: Union[str, NotGiven] = NOT_GIVEN,
        speed: Union[float, NotGiven] = NOT_GIVEN,
        stream: Union[bool, NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> Any:
        if stream is True:
            self.openai_client = self.openai_client.with_streaming_response
        extra_headers = kwargs.pop("extra_headers", {})
        response = self.openai_client.audio.speech.create(
            input=input,
            model=model,
            voice=voice,
            response_format=response_format,
            speed=speed,
            extra_headers=extra_headers,
            extra_body=kwargs,
        )

        return response


class AsyncAudio(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.transcriptions = AsyncTranscriptions(client)
        self.translations = AsyncTranslations(client)
        self.speech = AsyncSpeech(client)


class AsyncTranscriptions(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    @typing.no_type_check
    async def create(
        self,
        *,
        file: FileTypes,
        model: str,
        language: Union[str, NotGiven] = NOT_GIVEN,
        prompt: Union[str, NotGiven] = NOT_GIVEN,
        response_format: Union[str, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        timestamp_granularities: Union[List[str], NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> Transcription:
        extra_headers = kwargs.pop("extra_headers", {})
        response = (
            await self.openai_client.with_raw_response.audio.transcriptions.create(
                file=file,
                model=model,
                language=language,
                prompt=prompt,
                response_format=response_format,
                temperature=temperature,
                timestamp_granularities=timestamp_granularities,
                extra_headers=extra_headers,
                extra_body=kwargs,
            )
        )
        data = Transcription(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncTranslations(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(
        self,
        *,
        file: FileTypes,
        model: str,
        prompt: Union[str, NotGiven] = NOT_GIVEN,
        response_format: Union[str, NotGiven] = NOT_GIVEN,
        temperature: Union[float, NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> Translation:
        extra_headers = kwargs.pop("extra_headers", {})
        response = await self.openai_client.with_raw_response.audio.translations.create(
            file=file,
            model=model,
            prompt=prompt,
            response_format=response_format,
            temperature=temperature,
            extra_headers=extra_headers,
            extra_body=kwargs,
        )
        data = Translation(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncSpeech(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    @typing.no_type_check
    async def create(
        self,
        *,
        input: str,
        model: str,
        voice: str,
        response_format: Union[str, NotGiven] = NOT_GIVEN,
        speed: Union[float, NotGiven] = NOT_GIVEN,
        stream: Union[bool, NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> Any:
        if stream is True:
            self.openai_client = await self.openai_client.with_streaming_response
        extra_headers = kwargs.pop("extra_headers", {})
        response = await self.openai_client.audio.speech.create(
            input=input,
            model=model,
            voice=voice,
            response_format=response_format,
            speed=speed,
            extra_headers=extra_headers,
            extra_body=kwargs,
        )

        data = response

        return data



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypeAlias

from .assistant_tool_choice_param import AssistantToolChoiceParam

__all__ = ["AssistantToolChoiceOptionParam"]

AssistantToolChoiceOptionParam: TypeAlias = Union[Literal["none", "auto", "required"], AssistantToolChoiceParam]



import json
from typing import Any, Union
from unbound._vendor.openai._types import NOT_GIVEN, NotGiven
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.client import AsyncUnbound, Unbound
from unbound.api_resources.types.main_file_type import (
    FileDeleted,
    FileList,
    FileObject,
)


class MainFiles(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def create(self, file, purpose, **kwargs) -> FileObject:
        extra_headers = kwargs.pop("extra_headers", {})
        response = self.openai_client.with_raw_response.files.create(
            file=file, purpose=purpose, extra_body=kwargs, extra_headers=extra_headers
        )
        data = FileObject(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list(self, purpose: Union[str, NotGiven] = NOT_GIVEN, **kwargs) -> FileList:
        response = self.openai_client.with_raw_response.files.list(
            purpose=purpose, **kwargs
        )
        data = FileList(**json.loads(response.text))
        data._headers = response.headers

        return data

    def retrieve(self, file_id, **kwargs) -> FileObject:
        response = self.openai_client.with_raw_response.files.retrieve(
            file_id=file_id, extra_body=kwargs
        )
        data = FileObject(**json.loads(response.text))
        data._headers = response.headers

        return data

    def delete(self, file_id, **kwargs) -> FileDeleted:
        response = self.openai_client.with_raw_response.files.delete(
            file_id=file_id, extra_body=kwargs
        )
        data = FileDeleted(**json.loads(response.text))
        data._headers = response.headers

        return data

    def content(self, file_id, **kwargs) -> Any:
        response = self.openai_client.files.content(file_id=file_id, extra_body=kwargs)
        return response

    def retrieve_content(self, file_id, **kwargs) -> Any:
        response = self.openai_client.files.content(file_id=file_id, extra_body=kwargs)
        return response

    def wait_for_processing(
        self,
        id: str,
        *,
        poll_interval: float = 5.0,
        max_wait_seconds: float = 30 * 60,
    ) -> Any:
        response = self.openai_client.files.wait_for_processing(
            id=id,
            poll_interval=poll_interval,
            max_wait_seconds=max_wait_seconds,
        )
        return response


class AsyncMainFiles(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def create(self, file, purpose, **kwargs) -> FileObject:
        extra_headers = kwargs.pop("extra_headers", {})
        response = await self.openai_client.with_raw_response.files.create(
            file=file, purpose=purpose, extra_body=kwargs, extra_headers=extra_headers
        )
        data = FileObject(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list(
        self, purpose: Union[str, NotGiven] = NOT_GIVEN, **kwargs
    ) -> FileList:
        response = await self.openai_client.with_raw_response.files.list(
            purpose=purpose, **kwargs
        )
        data = FileList(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def retrieve(self, file_id, **kwargs) -> FileObject:
        response = await self.openai_client.with_raw_response.files.retrieve(
            file_id=file_id, extra_body=kwargs
        )
        data = FileObject(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def delete(self, file_id, **kwargs) -> FileDeleted:
        response = await self.openai_client.with_raw_response.files.delete(
            file_id=file_id, extra_body=kwargs
        )
        data = FileDeleted(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def content(self, file_id, **kwargs) -> Any:
        response = await self.openai_client.files.content(
            file_id=file_id, extra_body=kwargs
        )
        return response

    async def retrieve_content(self, file_id, **kwargs) -> Any:
        response = await self.openai_client.files.content(
            file_id=file_id, extra_body=kwargs
        )
        return response

    async def wait_for_processing(
        self,
        id: str,
        *,
        poll_interval: float = 5.0,
        max_wait_seconds: float = 30 * 60,
    ) -> Any:
        response = await self.openai_client.files.wait_for_processing(
            id=id,
            poll_interval=poll_interval,
            max_wait_seconds=max_wait_seconds,
        )
        return response



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .translation import Translation as Translation
from .speech_model import SpeechModel as SpeechModel
from .transcription import Transcription as Transcription
from .speech_create_params import SpeechCreateParams as SpeechCreateParams
from .translation_create_params import TranslationCreateParams as TranslationCreateParams
from .transcription_create_params import TranscriptionCreateParams as TranscriptionCreateParams



import json
import os
from typing import Optional
import requests

from unbound.api_resources.global_constants import UNBOUND_BASE_URL


class Logger:
    def __init__(
        self,
        api_key: Optional[str] = None,
    ) -> None:
        api_key = api_key or os.getenv("UNBOUND_API_KEY")
        if api_key is None:
            raise ValueError("API key is required to use the Logger API")

        self.headers = {
            "Content-Type": "application/json",
            "x-unbound-api-key": api_key,
        }

        self.url = UNBOUND_BASE_URL + "/logs"

    def log(
        self,
        log_object,
    ):
        response = requests.post(
            url=self.url, data=json.dumps(log_object), headers=self.headers
        )

        return response



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.


from .._models import BaseModel

__all__ = ["CompletionUsage"]


class CompletionUsage(BaseModel):
    completion_tokens: int
    """Number of tokens in the generated completion."""

    prompt_tokens: int
    """Number of tokens in the prompt."""

    total_tokens: int
    """Total number of tokens used in the request (prompt + completion)."""



from .chat_complete import ChatCompletion, AsyncChatCompletion
from .complete import Completion, AsyncCompletion
from .generation import Generations, AsyncGenerations, Prompts, AsyncPrompts
from .feedback import Feedback, AsyncFeedback
from .create_headers import createHeaders
from .post import Post, AsyncPost
from .embeddings import Embeddings, AsyncEmbeddings
from .images import Images, AsyncImages
from .assistants import Assistants, AsyncAssistants
from .threads import (
    Threads,
    Messages,
    Runs,
    Steps,
    AsyncThreads,
    AsyncMessages,
    AsyncRuns,
    AsyncSteps,
)
from .main_files import MainFiles, AsyncMainFiles
from .models import Models, AsyncModels
from .moderations import Moderations, AsyncModerations
from .audio import (
    Audio,
    Transcriptions,
    Translations,
    Speech,
    AsyncAudio,
    AsyncTranscriptions,
    AsyncTranslations,
    AsyncSpeech,
)
from .batches import Batches, AsyncBatches
from .fine_tuning import (
    FineTuning,
    Jobs,
    Checkpoints,
    AsyncFineTuning,
    AsyncJobs,
    AsyncCheckpoints,
)
from .vector_stores import (
    VectorStores,
    VectorFiles,
    VectorFileBatches,
    AsyncVectorStores,
    AsyncVectorFiles,
    AsyncVectorFileBatches,
)
from .admin import (
    Admin,
    Users,
    Invites,
    Workspaces,
    WorkspacesUsers,
    AsyncAdmin,
    AsyncUsers,
    AsyncInvites,
    AsyncWorkspaces,
    AsyncWorkspacesUsers,
)

from .beta_chat import (
    BetaChat,
    BetaCompletions,
    AsyncBetaChat,
    AsyncBetaCompletions,
)

from .uploads import (
    Uploads,
    Parts,
    AsyncUploads,
    AsyncParts,
)

from .configs import Configs, AsyncConfigs

from .api_keys import ApiKeys, AsyncApiKeys
from .virtual_keys import VirtualKeys, AsyncVirtualKeys
from .logs import Logs, AsyncLogs

__all__ = [
    "Completion",
    "AsyncCompletion",
    "ChatCompletion",
    "AsyncChatCompletion",
    "Generations",
    "AsyncGenerations",
    "Feedback",
    "AsyncFeedback",
    "Prompts",
    "AsyncPrompts",
    "createHeaders",
    "Post",
    "AsyncPost",
    "Embeddings",
    "AsyncEmbeddings",
    "Images",
    "AsyncImages",
    "Assistants",
    "AsyncAssistants",
    "MainFiles",
    "AsyncMainFiles",
    "Models",
    "AsyncModels",
    "Threads",
    "AsyncThreads",
    "Messages",
    "AsyncMessages",
    "Runs",
    "AsyncRuns",
    "Steps",
    "AsyncSteps",
    "Moderations",
    "AsyncModerations",
    "Audio",
    "Transcriptions",
    "Translations",
    "Speech",
    "AsyncAudio",
    "AsyncTranscriptions",
    "AsyncTranslations",
    "AsyncSpeech",
    "Batches",
    "AsyncBatches",
    "FineTuning",
    "Jobs",
    "Checkpoints",
    "AsyncFineTuning",
    "AsyncJobs",
    "AsyncCheckpoints",
    "VectorStores",
    "VectorFiles",
    "VectorFileBatches",
    "AsyncVectorStores",
    "AsyncVectorFiles",
    "AsyncVectorFileBatches",
    "Admin",
    "Users",
    "Invites",
    "Workspaces",
    "WorkspacesUsers",
    "AsyncAdmin",
    "AsyncUsers",
    "AsyncInvites",
    "AsyncWorkspaces",
    "AsyncWorkspacesUsers",
    "BetaChat",
    "BetaCompletions",
    "AsyncBetaChat",
    "AsyncBetaCompletions",
    "Uploads",
    "Parts",
    "AsyncUploads",
    "AsyncParts",
    "Configs",
    "AsyncConfigs",
    "ApiKeys",
    "AsyncApiKeys",
    "VirtualKeys",
    "AsyncVirtualKeys",
    "Logs",
    "AsyncLogs",
]



import json
from typing import Optional, Union
import typing
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.client import AsyncUnbound, Unbound
from unbound.api_resources.types.embeddings_type import CreateEmbeddingResponse
from ..._vendor.openai._types import NotGiven, NOT_GIVEN


class Embeddings(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    @typing.no_type_check
    def create(
        self,
        *,
        input: str,
        model: Optional[str] = "unbound-default",
        dimensions: Union[int, NotGiven] = NOT_GIVEN,
        encoding_format: Union[str, NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> CreateEmbeddingResponse:
        response = self.openai_client.with_raw_response.embeddings.create(
            input=input,
            model=model,
            dimensions=dimensions,
            encoding_format=encoding_format,
            user=user,
            extra_body=kwargs,
        )

        data = CreateEmbeddingResponse(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncEmbeddings(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    @typing.no_type_check
    async def create(
        self,
        *,
        input: str,
        model: Optional[str] = "unbound-default",
        dimensions: Union[int, NotGiven] = NOT_GIVEN,
        encoding_format: Union[str, NotGiven] = NOT_GIVEN,
        user: Union[str, NotGiven] = NOT_GIVEN,
        **kwargs
    ) -> CreateEmbeddingResponse:
        response = await self.openai_client.with_raw_response.embeddings.create(
            input=input,
            model=model,
            dimensions=dimensions,
            encoding_format=encoding_format,
            user=user,
            extra_body=kwargs,
        )
        data = CreateEmbeddingResponse(**json.loads(response.text))
        data._headers = response.headers

        return data



import json
from typing import Dict, Optional, Union
import httpx
from .utils import parse_headers
from typing import List, Any
from pydantic import BaseModel, PrivateAttr
from ..._vendor.openai.types.fine_tuning import FineTuningJobWandbIntegrationObject

__all__ = [
    "Error",
    "Hyperparameters",
    "FineTuningJob",
    "FineTuningJobList",
    "FineTuningJobEvent",
    "FineTuningJobEventList",
    "Metrics",
    "FineTuningJobCheckpoint",
    "FineTuningJobCheckpointList",
]


class Error(BaseModel):
    code: str
    message: str
    param: Optional[str] = None


class Hyperparameters(BaseModel):
    n_epochs: Union[str, int]


class FineTuningJob(BaseModel):
    id: str
    created_at: int
    error: Optional[Error] = None
    fine_tuned_model: Optional[str] = None
    finished_at: Optional[int] = None
    hyperparameters: Hyperparameters
    model: str
    object: str
    organization_id: str
    result_files: List[str]
    seed: int
    status: str
    trained_tokens: Optional[int] = None
    training_file: str
    validation_file: Optional[str] = None
    estimated_finish: Optional[int] = None
    integrations: Optional[List[FineTuningJobWandbIntegrationObject]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class FineTuningJobList(BaseModel):
    object: Optional[str] = None
    data: Optional[List[FineTuningJob]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class FineTuningJobEvent(BaseModel):
    id: str
    created_at: int
    level: str
    message: str
    object: str
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class FineTuningJobEventList(BaseModel):
    object: Optional[str] = None
    data: Optional[List[FineTuningJobEvent]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class Metrics(BaseModel):
    full_valid_loss: Optional[float] = None
    full_valid_mean_token_accuracy: Optional[float] = None
    step: Optional[float] = None
    train_loss: Optional[float] = None
    train_mean_token_accuracy: Optional[float] = None
    valid_loss: Optional[float] = None
    valid_mean_token_accuracy: Optional[float] = None


class FineTuningJobCheckpoint(BaseModel):
    id: str
    created_at: int
    fine_tuned_model_checkpoint: str
    fine_tuning_job_id: str
    metrics: Metrics
    object: str
    step_number: int
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class FineTuningJobCheckpointList(BaseModel):
    object: Optional[str] = None
    data: Optional[List[FineTuningJobCheckpoint]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from .beta import (
    Beta,
    AsyncBeta,
    BetaWithRawResponse,
    AsyncBetaWithRawResponse,
    BetaWithStreamingResponse,
    AsyncBetaWithStreamingResponse,
)
from .threads import (
    Threads,
    AsyncThreads,
    ThreadsWithRawResponse,
    AsyncThreadsWithRawResponse,
    ThreadsWithStreamingResponse,
    AsyncThreadsWithStreamingResponse,
)
from .assistants import (
    Assistants,
    AsyncAssistants,
    AssistantsWithRawResponse,
    AsyncAssistantsWithRawResponse,
    AssistantsWithStreamingResponse,
    AsyncAssistantsWithStreamingResponse,
)
from .vector_stores import (
    VectorStores,
    AsyncVectorStores,
    VectorStoresWithRawResponse,
    AsyncVectorStoresWithRawResponse,
    VectorStoresWithStreamingResponse,
    AsyncVectorStoresWithStreamingResponse,
)

__all__ = [
    "VectorStores",
    "AsyncVectorStores",
    "VectorStoresWithRawResponse",
    "AsyncVectorStoresWithRawResponse",
    "VectorStoresWithStreamingResponse",
    "AsyncVectorStoresWithStreamingResponse",
    "Assistants",
    "AsyncAssistants",
    "AssistantsWithRawResponse",
    "AsyncAssistantsWithRawResponse",
    "AssistantsWithStreamingResponse",
    "AsyncAssistantsWithStreamingResponse",
    "Threads",
    "AsyncThreads",
    "ThreadsWithRawResponse",
    "AsyncThreadsWithRawResponse",
    "ThreadsWithStreamingResponse",
    "AsyncThreadsWithStreamingResponse",
    "Beta",
    "AsyncBeta",
    "BetaWithRawResponse",
    "AsyncBetaWithRawResponse",
    "BetaWithStreamingResponse",
    "AsyncBetaWithStreamingResponse",
]



import json
from typing import Iterable, Optional, Union
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.client import AsyncUnbound, Unbound
from ..._vendor.openai._types import NotGiven, NOT_GIVEN
from ..._vendor.openai.types.fine_tuning import job_create_params

from unbound.api_resources.types.fine_tuning_type import (
    FineTuningJob,
    FineTuningJobCheckpointList,
    FineTuningJobEventList,
    FineTuningJobList,
)


class FineTuning(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.jobs = Jobs(client)


class Jobs(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.checkpoints = Checkpoints(client)

    def create(
        self,
        *,
        model: str,
        training_file: str,
        hyperparameters: Union[job_create_params.Hyperparameters, NotGiven] = NOT_GIVEN,
        integrations: Union[
            Optional[Iterable[job_create_params.Integration]], NotGiven
        ] = NOT_GIVEN,
        seed: Union[Optional[int], NotGiven] = NOT_GIVEN,
        suffix: Union[Optional[str], NotGiven] = NOT_GIVEN,
        validation_file: Union[Optional[str], NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJob:
        response = self.openai_client.with_raw_response.fine_tuning.jobs.create(
            model=model,
            training_file=training_file,
            hyperparameters=hyperparameters,
            integrations=integrations,
            seed=seed,
            suffix=suffix,
            validation_file=validation_file,
            extra_body=kwargs,
        )
        data = FineTuningJob(**json.loads(response.text))
        data._headers = response.headers

        return data

    def retrieve(self, fine_tuning_job_id: str, **kwargs) -> FineTuningJob:
        response = self.openai_client.with_raw_response.fine_tuning.jobs.retrieve(
            fine_tuning_job_id=fine_tuning_job_id, extra_body=kwargs
        )
        data = FineTuningJob(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list(
        self,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJobList:
        response = self.openai_client.with_raw_response.fine_tuning.jobs.list(
            after=after, limit=limit, **kwargs
        )
        data = FineTuningJobList(**json.loads(response.text))
        data._headers = response.headers

        return data

    def cancel(self, fine_tuning_job_id: str, **kwargs) -> FineTuningJob:
        response = self.openai_client.with_raw_response.fine_tuning.jobs.cancel(
            fine_tuning_job_id=fine_tuning_job_id, extra_body=kwargs
        )
        data = FineTuningJob(**json.loads(response.text))
        data._headers = response.headers

        return data

    def list_events(
        self,
        fine_tuning_job_id: str,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJobEventList:
        response = self.openai_client.with_raw_response.fine_tuning.jobs.list_events(
            fine_tuning_job_id=fine_tuning_job_id,
            after=after,
            limit=limit,
            extra_body=kwargs,
        )
        data = FineTuningJobEventList(**json.loads(response.text))
        data._headers = response.headers

        return data


class Checkpoints(APIResource):
    def __init__(self, client: Unbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    def list(
        self,
        fine_tuning_job_id: str,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJobCheckpointList:
        response = (
            self.openai_client.with_raw_response.fine_tuning.jobs.checkpoints.list(
                fine_tuning_job_id=fine_tuning_job_id,
                after=after,
                limit=limit,
                **kwargs,
            )
        )

        data = FineTuningJobCheckpointList(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncFineTuning(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.jobs = AsyncJobs(client)


class AsyncJobs(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client
        self.checkpoints = AsyncCheckpoints(client)

    async def create(
        self,
        *,
        model: str,
        training_file: str,
        hyperparameters: Union[job_create_params.Hyperparameters, NotGiven] = NOT_GIVEN,
        integrations: Union[
            Optional[Iterable[job_create_params.Integration]], NotGiven
        ] = NOT_GIVEN,
        seed: Union[Optional[int], NotGiven] = NOT_GIVEN,
        suffix: Union[Optional[str], NotGiven] = NOT_GIVEN,
        validation_file: Union[Optional[str], NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJob:
        response = await self.openai_client.with_raw_response.fine_tuning.jobs.create(
            model=model,
            training_file=training_file,
            hyperparameters=hyperparameters,
            integrations=integrations,
            seed=seed,
            suffix=suffix,
            validation_file=validation_file,
            extra_body=kwargs,
        )
        data = FineTuningJob(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def retrieve(self, fine_tuning_job_id: str, **kwargs) -> FineTuningJob:
        response = await self.openai_client.with_raw_response.fine_tuning.jobs.retrieve(
            fine_tuning_job_id=fine_tuning_job_id, extra_body=kwargs
        )
        data = FineTuningJob(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list(
        self,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJobList:
        response = await self.openai_client.with_raw_response.fine_tuning.jobs.list(
            after=after, limit=limit, **kwargs
        )
        data = FineTuningJobList(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def cancel(self, fine_tuning_job_id: str, **kwargs) -> FineTuningJob:
        response = await self.openai_client.with_raw_response.fine_tuning.jobs.cancel(
            fine_tuning_job_id, extra_body=kwargs
        )
        data = FineTuningJob(**json.loads(response.text))
        data._headers = response.headers

        return data

    async def list_events(
        self,
        fine_tuning_job_id: str,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJobEventList:
        response = (
            await self.openai_client.with_raw_response.fine_tuning.jobs.list_events(
                fine_tuning_job_id=fine_tuning_job_id,
                after=after,
                limit=limit,
                extra_body=kwargs,
            )
        )
        data = FineTuningJobEventList(**json.loads(response.text))
        data._headers = response.headers

        return data


class AsyncCheckpoints(AsyncAPIResource):
    def __init__(self, client: AsyncUnbound) -> None:
        super().__init__(client)
        self.openai_client = client.openai_client

    async def list(
        self,
        fine_tuning_job_id: str,
        *,
        after: Union[str, NotGiven] = NOT_GIVEN,
        limit: Union[int, NotGiven] = NOT_GIVEN,
        **kwargs,
    ) -> FineTuningJobCheckpointList:
        response = await self.openai_client.with_raw_response.fine_tuning.jobs.checkpoints.list(  # noqa: E501
            fine_tuning_job_id=fine_tuning_job_id,
            after=after,
            limit=limit,
            **kwargs,
        )

        data = FineTuningJobCheckpointList(**json.loads(response.text))
        data._headers = response.headers

        return data



from typing import Any, Dict, List, Optional, Union
from unbound.api_resources.base_client import APIClient, AsyncAPIClient
from urllib.parse import urlencode
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.types.api_keys_type import (
    ApiKeyAddResponse,
    ApiKeyGetResponse,
    ApiKeyListResponse,
)
from unbound.api_resources.utils import GenericResponse
from unbound.api_resources.utils import UnboundApiPaths


class ApiKeys(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    def create(
        self,
        *,
        type: Optional[str] = None,
        sub_type: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        workspace_id: Optional[str] = None,
        user_id: Optional[str] = None,
        rate_limits: Optional[List[Dict[str, Any]]] = None,
        usage_limits: Optional[Dict[str, Any]] = None,
        scopes: List[str],
        defaults: Optional[Dict[str, Any]] = None,
    ) -> ApiKeyAddResponse:
        body = {
            "type": type,
            "sub-type": sub_type,
            "name": name,
            "description": description,
            "workspace_id": workspace_id,
            "user_id": user_id,
            "rate_limits": rate_limits,
            "usage_limits": usage_limits,
            "scopes": scopes,
            "defaults": defaults,
        }
        return self._post(
            f"{UnboundApiPaths.API_KEYS_API}/{type}/{sub_type}",
            body=body,
            params=None,
            cast_to=ApiKeyAddResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def retrieve(self, *, id: Optional[str]) -> ApiKeyGetResponse:
        return self._get(
            f"{UnboundApiPaths.API_KEYS_API}/{id}",
            params=None,
            body=None,
            cast_to=ApiKeyGetResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def list(
        self,
        *,
        page_size: Optional[Union[int, str]] = None,
        current_page: Optional[int] = 0,
        workspace_id: Optional[str] = None,
    ) -> ApiKeyListResponse:
        query = {
            "page_size": page_size,
            "current_page": current_page,
            "workspace_id": workspace_id,
        }
        filtered_query = {k: v for k, v in query.items() if v is not None}
        query_string = urlencode(filtered_query)
        return self._get(
            f"{UnboundApiPaths.API_KEYS_API}?{query_string}",
            params=None,
            body=None,
            cast_to=ApiKeyListResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def update(
        self,
        *,
        id: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        rate_limits: Optional[List[Dict[str, Any]]] = None,
        usage_limits: Optional[Dict[str, Any]] = None,
        scopes: Optional[List[str]] = None,
        defaults: Optional[Dict[str, Any]] = None,
    ) -> Any:
        body = {
            "id": id,
            "name": name,
            "description": description,
            "rate_limits": rate_limits,
            "usage_limits": usage_limits,
            "scopes": scopes,
            "defaults": defaults,
        }
        return self._put(
            f"{UnboundApiPaths.API_KEYS_API}/{id}",
            body=body,
            params=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    def delete(
        self,
        *,
        id: Optional[str],
    ) -> Any:
        return self._delete(
            f"{UnboundApiPaths.API_KEYS_API}/{id}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )


class AsyncApiKeys(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    async def create(
        self,
        *,
        type: Optional[str] = None,
        sub_type: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        workspace_id: Optional[str] = None,
        user_id: Optional[str] = None,
        rate_limits: Optional[List[Dict[str, Any]]] = None,
        usage_limits: Optional[Dict[str, Any]] = None,
        scopes: List[str],
        defaults: Optional[Dict[str, Any]] = None,
    ) -> ApiKeyAddResponse:
        body = {
            "type": type,
            "sub-type": sub_type,
            "name": name,
            "description": description,
            "workspace_id": workspace_id,
            "user_id": user_id,
            "rate_limits": rate_limits,
            "usage_limits": usage_limits,
            "scopes": scopes,
            "defaults": defaults,
        }
        return await self._post(
            f"{UnboundApiPaths.API_KEYS_API}/{type}/{sub_type}",
            body=body,
            params=None,
            cast_to=ApiKeyAddResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def retrieve(self, *, id: Optional[str]) -> ApiKeyGetResponse:
        return await self._get(
            f"{UnboundApiPaths.API_KEYS_API}/{id}",
            params=None,
            body=None,
            cast_to=ApiKeyGetResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def list(
        self,
        *,
        page_size: Optional[Union[int, str]] = None,
        current_page: Optional[int] = 0,
        workspace_id: Optional[str] = None,
    ) -> ApiKeyListResponse:
        query = {
            "page_size": page_size,
            "current_page": current_page,
            "workspace_id": workspace_id,
        }
        filtered_query = {k: v for k, v in query.items() if v is not None}
        query_string = urlencode(filtered_query)
        return await self._get(
            f"{UnboundApiPaths.API_KEYS_API}?{query_string}",
            params=None,
            body=None,
            cast_to=ApiKeyListResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def update(
        self,
        *,
        id: Optional[str] = None,
        name: Optional[str] = None,
        description: Optional[str] = None,
        rate_limits: Optional[List[Dict[str, Any]]] = None,
        usage_limits: Optional[Dict[str, Any]] = None,
        scopes: Optional[List[str]] = None,
        defaults: Optional[Dict[str, Any]] = None,
    ) -> Any:
        body = {
            "id": id,
            "name": name,
            "description": description,
            "rate_limits": rate_limits,
            "usage_limits": usage_limits,
            "scopes": scopes,
            "defaults": defaults,
        }
        return await self._put(
            f"{UnboundApiPaths.API_KEYS_API}/{id}",
            body=body,
            params=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )

    async def delete(
        self,
        *,
        id: Optional[str],
    ) -> Any:
        return await self._delete(
            f"{UnboundApiPaths.API_KEYS_API}/{id}",
            params=None,
            body=None,
            cast_to=GenericResponse,
            stream=False,
            stream_cls=None,
            headers={},
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import List, Union, Mapping, cast
from typing_extensions import Literal

import httpx

from ... import _legacy_response
from ..._types import NOT_GIVEN, Body, Query, Headers, NotGiven, FileTypes
from ..._utils import (
    extract_files,
    maybe_transform,
    deepcopy_minimal,
    async_maybe_transform,
)
from ..._compat import cached_property
from ..._resource import SyncAPIResource, AsyncAPIResource
from ..._response import to_streamed_response_wrapper, async_to_streamed_response_wrapper
from ...types.audio import transcription_create_params
from ..._base_client import make_request_options
from ...types.audio_model import AudioModel
from ...types.audio.transcription import Transcription

__all__ = ["Transcriptions", "AsyncTranscriptions"]


class Transcriptions(SyncAPIResource):
    @cached_property
    def with_raw_response(self) -> TranscriptionsWithRawResponse:
        return TranscriptionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> TranscriptionsWithStreamingResponse:
        return TranscriptionsWithStreamingResponse(self)

    def create(
        self,
        *,
        file: FileTypes,
        model: Union[str, AudioModel],
        language: str | NotGiven = NOT_GIVEN,
        prompt: str | NotGiven = NOT_GIVEN,
        response_format: Literal["json", "text", "srt", "verbose_json", "vtt"] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        timestamp_granularities: List[Literal["word", "segment"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Transcription:
        """
        Transcribes audio into the input language.

        Args:
          file:
              The audio file object (not file name) to transcribe, in one of these formats:
              flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm.

          model: ID of the model to use. Only `whisper-1` (which is powered by our open source
              Whisper V2 model) is currently available.

          language: The language of the input audio. Supplying the input language in
              [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes) format will
              improve accuracy and latency.

          prompt: An optional text to guide the model's style or continue a previous audio
              segment. The
              [prompt](https://platform.openai.com/docs/guides/speech-to-text/prompting)
              should match the audio language.

          response_format: The format of the transcript output, in one of these options: `json`, `text`,
              `srt`, `verbose_json`, or `vtt`.

          temperature: The sampling temperature, between 0 and 1. Higher values like 0.8 will make the
              output more random, while lower values like 0.2 will make it more focused and
              deterministic. If set to 0, the model will use
              [log probability](https://en.wikipedia.org/wiki/Log_probability) to
              automatically increase the temperature until certain thresholds are hit.

          timestamp_granularities: The timestamp granularities to populate for this transcription.
              `response_format` must be set `verbose_json` to use timestamp granularities.
              Either or both of these options are supported: `word`, or `segment`. Note: There
              is no additional latency for segment timestamps, but generating word timestamps
              incurs additional latency.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "model": model,
                "language": language,
                "prompt": prompt,
                "response_format": response_format,
                "temperature": temperature,
                "timestamp_granularities": timestamp_granularities,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return self._post(
            "/audio/transcriptions",
            body=maybe_transform(body, transcription_create_params.TranscriptionCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transcription,
        )


class AsyncTranscriptions(AsyncAPIResource):
    @cached_property
    def with_raw_response(self) -> AsyncTranscriptionsWithRawResponse:
        return AsyncTranscriptionsWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncTranscriptionsWithStreamingResponse:
        return AsyncTranscriptionsWithStreamingResponse(self)

    async def create(
        self,
        *,
        file: FileTypes,
        model: Union[str, AudioModel],
        language: str | NotGiven = NOT_GIVEN,
        prompt: str | NotGiven = NOT_GIVEN,
        response_format: Literal["json", "text", "srt", "verbose_json", "vtt"] | NotGiven = NOT_GIVEN,
        temperature: float | NotGiven = NOT_GIVEN,
        timestamp_granularities: List[Literal["word", "segment"]] | NotGiven = NOT_GIVEN,
        # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
        # The extra values given here take precedence over values defined on the client or passed to this method.
        extra_headers: Headers | None = None,
        extra_query: Query | None = None,
        extra_body: Body | None = None,
        timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,
    ) -> Transcription:
        """
        Transcribes audio into the input language.

        Args:
          file:
              The audio file object (not file name) to transcribe, in one of these formats:
              flac, mp3, mp4, mpeg, mpga, m4a, ogg, wav, or webm.

          model: ID of the model to use. Only `whisper-1` (which is powered by our open source
              Whisper V2 model) is currently available.

          language: The language of the input audio. Supplying the input language in
              [ISO-639-1](https://en.wikipedia.org/wiki/List_of_ISO_639-1_codes) format will
              improve accuracy and latency.

          prompt: An optional text to guide the model's style or continue a previous audio
              segment. The
              [prompt](https://platform.openai.com/docs/guides/speech-to-text/prompting)
              should match the audio language.

          response_format: The format of the transcript output, in one of these options: `json`, `text`,
              `srt`, `verbose_json`, or `vtt`.

          temperature: The sampling temperature, between 0 and 1. Higher values like 0.8 will make the
              output more random, while lower values like 0.2 will make it more focused and
              deterministic. If set to 0, the model will use
              [log probability](https://en.wikipedia.org/wiki/Log_probability) to
              automatically increase the temperature until certain thresholds are hit.

          timestamp_granularities: The timestamp granularities to populate for this transcription.
              `response_format` must be set `verbose_json` to use timestamp granularities.
              Either or both of these options are supported: `word`, or `segment`. Note: There
              is no additional latency for segment timestamps, but generating word timestamps
              incurs additional latency.

          extra_headers: Send extra headers

          extra_query: Add additional query parameters to the request

          extra_body: Add additional JSON properties to the request

          timeout: Override the client-level default timeout for this request, in seconds
        """
        body = deepcopy_minimal(
            {
                "file": file,
                "model": model,
                "language": language,
                "prompt": prompt,
                "response_format": response_format,
                "temperature": temperature,
                "timestamp_granularities": timestamp_granularities,
            }
        )
        files = extract_files(cast(Mapping[str, object], body), paths=[["file"]])
        # It should be noted that the actual Content-Type header that will be
        # sent to the server will contain a `boundary` parameter, e.g.
        # multipart/form-data; boundary=---abc--
        extra_headers = {"Content-Type": "multipart/form-data", **(extra_headers or {})}
        return await self._post(
            "/audio/transcriptions",
            body=await async_maybe_transform(body, transcription_create_params.TranscriptionCreateParams),
            files=files,
            options=make_request_options(
                extra_headers=extra_headers, extra_query=extra_query, extra_body=extra_body, timeout=timeout
            ),
            cast_to=Transcription,
        )


class TranscriptionsWithRawResponse:
    def __init__(self, transcriptions: Transcriptions) -> None:
        self._transcriptions = transcriptions

        self.create = _legacy_response.to_raw_response_wrapper(
            transcriptions.create,
        )


class AsyncTranscriptionsWithRawResponse:
    def __init__(self, transcriptions: AsyncTranscriptions) -> None:
        self._transcriptions = transcriptions

        self.create = _legacy_response.async_to_raw_response_wrapper(
            transcriptions.create,
        )


class TranscriptionsWithStreamingResponse:
    def __init__(self, transcriptions: Transcriptions) -> None:
        self._transcriptions = transcriptions

        self.create = to_streamed_response_wrapper(
            transcriptions.create,
        )


class AsyncTranscriptionsWithStreamingResponse:
    def __init__(self, transcriptions: AsyncTranscriptions) -> None:
        self._transcriptions = transcriptions

        self.create = async_to_streamed_response_wrapper(
            transcriptions.create,
        )



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from .threads import (
    Threads,
    AsyncThreads,
    ThreadsWithRawResponse,
    AsyncThreadsWithRawResponse,
    ThreadsWithStreamingResponse,
    AsyncThreadsWithStreamingResponse,
)
from ..._compat import cached_property
from .chat.chat import Chat, AsyncChat
from .assistants import (
    Assistants,
    AsyncAssistants,
    AssistantsWithRawResponse,
    AsyncAssistantsWithRawResponse,
    AssistantsWithStreamingResponse,
    AsyncAssistantsWithStreamingResponse,
)
from ..._resource import SyncAPIResource, AsyncAPIResource
from .vector_stores import (
    VectorStores,
    AsyncVectorStores,
    VectorStoresWithRawResponse,
    AsyncVectorStoresWithRawResponse,
    VectorStoresWithStreamingResponse,
    AsyncVectorStoresWithStreamingResponse,
)
from .threads.threads import Threads, AsyncThreads
from .vector_stores.vector_stores import VectorStores, AsyncVectorStores

__all__ = ["Beta", "AsyncBeta"]


class Beta(SyncAPIResource):
    @cached_property
    def chat(self) -> Chat:
        return Chat(self._client)

    @cached_property
    def vector_stores(self) -> VectorStores:
        return VectorStores(self._client)

    @cached_property
    def assistants(self) -> Assistants:
        return Assistants(self._client)

    @cached_property
    def threads(self) -> Threads:
        return Threads(self._client)

    @cached_property
    def with_raw_response(self) -> BetaWithRawResponse:
        return BetaWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> BetaWithStreamingResponse:
        return BetaWithStreamingResponse(self)


class AsyncBeta(AsyncAPIResource):
    @cached_property
    def chat(self) -> AsyncChat:
        return AsyncChat(self._client)

    @cached_property
    def vector_stores(self) -> AsyncVectorStores:
        return AsyncVectorStores(self._client)

    @cached_property
    def assistants(self) -> AsyncAssistants:
        return AsyncAssistants(self._client)

    @cached_property
    def threads(self) -> AsyncThreads:
        return AsyncThreads(self._client)

    @cached_property
    def with_raw_response(self) -> AsyncBetaWithRawResponse:
        return AsyncBetaWithRawResponse(self)

    @cached_property
    def with_streaming_response(self) -> AsyncBetaWithStreamingResponse:
        return AsyncBetaWithStreamingResponse(self)


class BetaWithRawResponse:
    def __init__(self, beta: Beta) -> None:
        self._beta = beta

    @cached_property
    def vector_stores(self) -> VectorStoresWithRawResponse:
        return VectorStoresWithRawResponse(self._beta.vector_stores)

    @cached_property
    def assistants(self) -> AssistantsWithRawResponse:
        return AssistantsWithRawResponse(self._beta.assistants)

    @cached_property
    def threads(self) -> ThreadsWithRawResponse:
        return ThreadsWithRawResponse(self._beta.threads)


class AsyncBetaWithRawResponse:
    def __init__(self, beta: AsyncBeta) -> None:
        self._beta = beta

    @cached_property
    def vector_stores(self) -> AsyncVectorStoresWithRawResponse:
        return AsyncVectorStoresWithRawResponse(self._beta.vector_stores)

    @cached_property
    def assistants(self) -> AsyncAssistantsWithRawResponse:
        return AsyncAssistantsWithRawResponse(self._beta.assistants)

    @cached_property
    def threads(self) -> AsyncThreadsWithRawResponse:
        return AsyncThreadsWithRawResponse(self._beta.threads)


class BetaWithStreamingResponse:
    def __init__(self, beta: Beta) -> None:
        self._beta = beta

    @cached_property
    def vector_stores(self) -> VectorStoresWithStreamingResponse:
        return VectorStoresWithStreamingResponse(self._beta.vector_stores)

    @cached_property
    def assistants(self) -> AssistantsWithStreamingResponse:
        return AssistantsWithStreamingResponse(self._beta.assistants)

    @cached_property
    def threads(self) -> ThreadsWithStreamingResponse:
        return ThreadsWithStreamingResponse(self._beta.threads)


class AsyncBetaWithStreamingResponse:
    def __init__(self, beta: AsyncBeta) -> None:
        self._beta = beta

    @cached_property
    def vector_stores(self) -> AsyncVectorStoresWithStreamingResponse:
        return AsyncVectorStoresWithStreamingResponse(self._beta.vector_stores)

    @cached_property
    def assistants(self) -> AsyncAssistantsWithStreamingResponse:
        return AsyncAssistantsWithStreamingResponse(self._beta.assistants)

    @cached_property
    def threads(self) -> AsyncThreadsWithStreamingResponse:
        return AsyncThreadsWithStreamingResponse(self._beta.threads)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union, Optional
from typing_extensions import Literal, Required, TypedDict

from .._types import FileTypes
from .image_model import ImageModel

__all__ = ["ImageCreateVariationParams"]


class ImageCreateVariationParams(TypedDict, total=False):
    image: Required[FileTypes]
    """The image to use as the basis for the variation(s).

    Must be a valid PNG file, less than 4MB, and square.
    """

    model: Union[str, ImageModel, None]
    """The model to use for image generation.

    Only `dall-e-2` is supported at this time.
    """

    n: Optional[int]
    """The number of images to generate.

    Must be between 1 and 10. For `dall-e-3`, only `n=1` is supported.
    """

    response_format: Optional[Literal["url", "b64_json"]]
    """The format in which the generated images are returned.

    Must be one of `url` or `b64_json`. URLs are only valid for 60 minutes after the
    image has been generated.
    """

    size: Optional[Literal["256x256", "512x512", "1024x1024"]]
    """The size of the generated images.

    Must be one of `256x256`, `512x512`, or `1024x1024`.
    """

    user: str
    """
    A unique identifier representing your end-user, which can help OpenAI to monitor
    and detect abuse.
    [Learn more](https://platform.openai.com/docs/guides/safety-best-practices/end-user-ids).
    """



from typing import TypeVar, Union

import httpx

from unbound.api_resources.types.generation_type import (
    PromptCompletionChunk,
    PromptRender,
)
from .streaming import Stream, AsyncStream
from .utils import GenericResponse
from .types.chat_complete_type import ChatCompletionChunk
from .types.complete_type import TextCompletionChunk

StreamT = TypeVar(
    "StreamT",
    bound=Stream[
        Union[
            ChatCompletionChunk,
            TextCompletionChunk,
            GenericResponse,
            PromptCompletionChunk,
            PromptRender,
            httpx.Response,
        ]
    ],
)

AsyncStreamT = TypeVar(
    "AsyncStreamT",
    bound=AsyncStream[
        Union[
            ChatCompletionChunk,
            TextCompletionChunk,
            GenericResponse,
            PromptCompletionChunk,
            PromptRender,
            httpx.Response,
        ]
    ],
)



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing_extensions import Literal

from ...._models import BaseModel
from .message_delta import MessageDelta

__all__ = ["MessageDeltaEvent"]


class MessageDeltaEvent(BaseModel):
    id: str
    """The identifier of the message, which can be referenced in API endpoints."""

    delta: MessageDelta
    """The delta containing the fields that have changed on the Message."""

    object: Literal["thread.message.delta"]
    """The object type, which is always `thread.message.delta`."""



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from __future__ import annotations

from typing import Union
from typing_extensions import Literal, TypeAlias

from ...types import shared_params

__all__ = ["AssistantResponseFormatOptionParam"]

AssistantResponseFormatOptionParam: TypeAlias = Union[
    Literal["auto"],
    shared_params.ResponseFormatText,
    shared_params.ResponseFormatJSONObject,
    shared_params.ResponseFormatJSONSchema,
]



from typing import Union, overload, Literal

from unbound.api_resources.base_client import APIClient, AsyncAPIClient

from unbound.api_resources.streaming import Stream, AsyncStream
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.utils import GenericResponse


class Post(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    @overload
    def create(
        self,
        *,
        url: str,
        stream: Literal[True],
        **kwargs,
    ) -> Stream[GenericResponse]:
        ...

    @overload
    def create(
        self,
        *,
        url: str,
        stream: Literal[False] = False,
        **kwargs,
    ) -> GenericResponse:
        ...

    @overload
    def create(
        self,
        *,
        url: str,
        stream: bool = False,
        **kwargs,
    ) -> Union[GenericResponse, Stream[GenericResponse]]:
        ...

    def create(
        self,
        *,
        url: str,
        stream: bool = False,
        **kwargs,
    ) -> Union[GenericResponse, Stream[GenericResponse]]:
        files = kwargs.pop("files", None)
        headers = kwargs.pop("headers", {})
        return self._post(
            url,
            body=kwargs,
            files=files,
            params=None,
            cast_to=GenericResponse,
            stream_cls=Stream[GenericResponse],
            stream=stream,
            headers=headers,
        )


class AsyncPost(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    @overload
    async def create(
        self,
        *,
        url: str,
        stream: Literal[True],
        **kwargs,
    ) -> AsyncStream[GenericResponse]:
        ...

    @overload
    async def create(
        self,
        *,
        url: str,
        stream: Literal[False] = False,
        **kwargs,
    ) -> GenericResponse:
        ...

    @overload
    async def create(
        self,
        *,
        url: str,
        stream: bool = False,
        **kwargs,
    ) -> Union[GenericResponse, AsyncStream[GenericResponse]]:
        ...

    async def create(
        self,
        *,
        url: str,
        stream: bool = False,
        **kwargs,
    ) -> Union[GenericResponse, AsyncStream[GenericResponse]]:
        files = kwargs.pop("files", None)
        headers = kwargs.pop("headers", {})
        return await self._post(
            url,
            body=kwargs,
            files=files,
            params=None,
            cast_to=GenericResponse,
            stream_cls=AsyncStream[GenericResponse],
            stream=stream,
            headers=headers,
        )



from typing import Optional, Dict, Any, List
from unbound.api_resources.apis.api_resource import APIResource, AsyncAPIResource
from unbound.api_resources.base_client import APIClient, AsyncAPIClient
from unbound.api_resources.streaming import AsyncStream, Stream
from unbound.api_resources.types.feedback_type import FeedbackResponse
from unbound.api_resources.utils import UnboundApiPaths


class Feedback(APIResource):
    def __init__(self, client: APIClient) -> None:
        super().__init__(client)

    def create(
        self,
        *,
        trace_id: Optional[str] = None,
        value: Optional[int] = None,
        weight: Optional[float] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> FeedbackResponse:
        body = dict(trace_id=trace_id, value=value, weight=weight, metadata=metadata)
        return self._post(
            UnboundApiPaths.FEEDBACK_API,
            body=body,
            params=None,
            cast_to=FeedbackResponse,
            stream_cls=Stream[FeedbackResponse],
            stream=False,
            headers={},
        )

    def bulk_create(self, *, feedbacks: List[Dict[str, Any]]) -> FeedbackResponse:
        body = feedbacks
        return self._post(
            UnboundApiPaths.FEEDBACK_API,
            body=body,
            params=None,
            cast_to=FeedbackResponse,
            stream_cls=Stream[FeedbackResponse],
            stream=False,
            headers={},
        )

    def update(
        self,
        *,
        feedback_id: Optional[str] = None,
        value: Optional[int] = None,
        weight: Optional[float] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> FeedbackResponse:
        body = dict(value=value, weight=weight, metadata=metadata)

        return self._put(
            f"{UnboundApiPaths.FEEDBACK_API}/{feedback_id}",
            body=body,
            params=None,
            cast_to=FeedbackResponse,
            stream_cls=Stream[FeedbackResponse],
            stream=False,
            headers={},
        )


class AsyncFeedback(AsyncAPIResource):
    def __init__(self, client: AsyncAPIClient) -> None:
        super().__init__(client)

    async def create(
        self,
        *,
        trace_id: Optional[str] = None,
        value: Optional[int] = None,
        weight: Optional[float] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> FeedbackResponse:
        body = dict(trace_id=trace_id, value=value, weight=weight, metadata=metadata)
        return await self._post(
            UnboundApiPaths.FEEDBACK_API,
            body=body,
            params=None,
            cast_to=FeedbackResponse,
            stream_cls=AsyncStream[FeedbackResponse],
            stream=False,
            headers={},
        )

    async def bulk_create(self, *, feedbacks: List[Dict[str, Any]]) -> FeedbackResponse:
        body = feedbacks
        return await self._post(
            UnboundApiPaths.FEEDBACK_API,
            body=body,
            params=None,
            cast_to=FeedbackResponse,
            stream_cls=AsyncStream[FeedbackResponse],
            stream=False,
            headers={},
        )

    async def update(
        self,
        *,
        feedback_id: Optional[str] = None,
        value: Optional[int] = None,
        weight: Optional[float] = None,
        metadata: Optional[Dict[str, Any]] = None,
    ) -> FeedbackResponse:
        body = dict(value=value, weight=weight, metadata=metadata)
        return await self._put(
            f"{UnboundApiPaths.FEEDBACK_API}/{feedback_id}",
            body=body,
            params=None,
            cast_to=FeedbackResponse,
            stream_cls=AsyncStream[FeedbackResponse],
            stream=False,
            headers={},
        )



import json
import builtins
from typing import Dict, Optional
import httpx
from .utils import parse_headers
from typing import List, Any
from pydantic import BaseModel, PrivateAttr
from ..._vendor.openai.types.batch_error import BatchError
from ..._vendor.openai.types.batch_request_counts import BatchRequestCounts

__all__ = ["Batch", "BatchList", "Errors"]


class Errors(BaseModel):
    data: Optional[List[BatchError]] = None

    object: Optional[str] = None
    """The object type, which is always `list`."""


class Batch(BaseModel):
    id: str
    completion_window: str
    created_at: int
    endpoint: str
    input_file_id: str
    object: str
    status: str
    cancelled_at: Optional[int] = None
    cancelling_at: Optional[int] = None
    completed_at: Optional[int] = None
    error_file_id: Optional[str] = None
    errors: Optional[Errors] = None
    expired_at: Optional[int] = None
    expires_at: Optional[int] = None
    failed_at: Optional[int] = None
    finalizing_at: Optional[int] = None
    in_progress_at: Optional[int] = None
    metadata: Optional[builtins.object] = None
    output_file_id: Optional[str] = None
    request_counts: Optional[BatchRequestCounts] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class BatchList(BaseModel):
    object: Optional[str] = None
    data: Optional[List[Batch]] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)



import json
from typing import Dict, Literal, Optional, Union
import httpx
from .utils import parse_headers
from typing import List
from pydantic import BaseModel, PrivateAttr

__all__ = [
    "Run",
    "Usage",
    "LastError",
    "Function",
    "RequiredActionFunctionToolCall",
    "RequiredActionSubmitToolOutputs",
    "RequiredAction",
    "FunctionDefinition",
    "ToolAssistantToolsCode",
    "ToolAssistantToolsRetrieval",
    "ToolAssistantToolsFunction",
    "Tool",
    "RunList",
    "RunStep",
    "StepDetails",
    "ToolCallsStepDetails",
    "MessageCreationStepDetails",
    "MessageCreation",
    "ToolCall",
    "CodeInterpreter",
    "CodeInterpreterOutput",
    "CodeInterpreterOutputLogs",
    "CodeInterpreterOutputImage",
    "CodeInterpreterOutputImageImage",
    "CodeToolCall",
    "RetrievalToolCall",
    "FunctionToolCall",
    "FunctionParameters",
    "RunStepList",
]


class Function(BaseModel):
    arguments: Optional[str]
    name: Optional[str]
    output: Optional[str] = None


class FunctionToolCall(BaseModel):
    id: Optional[str]
    function: Function
    type: Literal["function"]


class RetrievalToolCall(BaseModel):
    id: Optional[str]
    retrieval: Optional[object]
    type: Optional[str]


class CodeInterpreterOutputLogs(BaseModel):
    logs: Optional[str]
    type: Optional[str]


class CodeInterpreterOutputImageImage(BaseModel):
    file_id: Optional[str]


class CodeInterpreterOutputImage(BaseModel):
    image: CodeInterpreterOutputImageImage
    type: Optional[str]


CodeInterpreterOutput = Union[CodeInterpreterOutputLogs, CodeInterpreterOutputImage]


class CodeInterpreter(BaseModel):
    input: Optional[str]
    outputs: List[CodeInterpreterOutput]


class CodeToolCall(BaseModel):
    id: Optional[str]
    code_interpreter: CodeInterpreter
    type: Optional[str]


ToolCall = Union[CodeToolCall, RetrievalToolCall, FunctionToolCall]


class ToolCallsStepDetails(BaseModel):
    tool_calls: Optional[List[ToolCall]]
    type: Optional[str]


class MessageCreation(BaseModel):
    message_id: Optional[str]


class MessageCreationStepDetails(BaseModel):
    message_creation: Optional[MessageCreation]

    type: Optional[str]


StepDetails = Union[MessageCreationStepDetails, ToolCallsStepDetails]


class Usage(BaseModel):
    completion_tokens: Optional[int]
    prompt_tokens: Optional[int]
    total_tokens: Optional[int]


class LastError(BaseModel):
    code: Optional[str]
    message: Optional[str]


class FunctionRA(BaseModel):
    arguments: Optional[str]
    name: Optional[str]


class RequiredActionFunctionToolCall(BaseModel):
    id: Optional[str]
    function: Optional[FunctionRA]
    type: Optional[str]


class RequiredActionSubmitToolOutputs(BaseModel):
    tool_calls: Optional[List[RequiredActionFunctionToolCall]]


class RequiredAction(BaseModel):
    submit_tool_outputs: Optional[RequiredActionSubmitToolOutputs]
    type: Optional[str]


FunctionParameters = Dict[str, object]


class FunctionDefinition(BaseModel):
    name: Optional[str]
    description: Optional[str] = None
    parameters: Optional[FunctionParameters] = None


class ToolAssistantToolsCode(BaseModel):
    type: Optional[str]


class ToolAssistantToolsRetrieval(BaseModel):
    type: Optional[str]


class ToolAssistantToolsFunction(BaseModel):
    function: Optional[FunctionDefinition]
    type: Optional[str]


Tool = Union[
    ToolAssistantToolsCode, ToolAssistantToolsRetrieval, ToolAssistantToolsFunction
]


class Run(BaseModel):
    id: Optional[str]
    assistant_id: Optional[str]
    cancelled_at: Optional[int] = None
    completed_at: Optional[int] = None
    created_at: Optional[int]
    expires_at: Optional[int]
    failed_at: Optional[int] = None
    file_ids: Optional[List[str]] = None
    instructions: Optional[str]
    last_error: Optional[LastError] = None
    metadata: Optional[object] = None
    model: Optional[str]
    object: Optional[str]
    required_action: Optional[RequiredAction] = None
    started_at: Optional[int] = None
    status: Optional[str]
    thread_id: Optional[str]
    tools: Optional[List[Tool]]
    usage: Optional[Usage] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class RunList(BaseModel, extra="allow"):
    object: Optional[str]
    data: Optional[List[Run]]
    first_id: Optional[str]
    last_id: Optional[str]
    has_more: Optional[bool]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class RunStep(BaseModel, extra="allow"):
    id: Optional[str]
    assistant_id: Optional[str]
    cancelled_at: Optional[int] = None
    completed_at: Optional[int] = None
    created_at: Optional[int]
    expired_at: Optional[int] = None
    failed_at: Optional[int] = None
    last_error: Optional[LastError] = None
    metadata: Optional[object] = None
    object: Optional[str]
    run_id: Optional[str]
    status: Optional[str]
    step_details: Optional[StepDetails]
    thread_id: Optional[str]
    type: Optional[str]
    usage: Optional[Usage] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class RunStepList(BaseModel, extra="allow"):
    object: Optional[str]
    data: Optional[List[RunStep]]
    first_id: Optional[str]
    last_id: Optional[str]
    has_more: Optional[bool]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)



from typing import Optional

from typing import List
from pydantic import BaseModel


class FeedbackResponse(BaseModel):
    status: Optional[str]
    message: Optional[str]
    feedback_ids: Optional[List[str]]



"""
Utility Tools for the Unbound Class.

This file module contains a collection of utility functions designed to enhance
the functionality and usability of the Unbound class
"""
from typing import TYPE_CHECKING


if TYPE_CHECKING:
    pass

IMPORT_ERROR_MESSAGE = (
    "Llama-Index is not installed.Please install it with `pip install llama-index`."
)


def all_available_models():
    try:
        from llama_index.llms.anthropic_utils import CLAUDE_MODELS
        from llama_index.llms.openai_utils import (
            AZURE_TURBO_MODELS,
            GPT3_5_MODELS,
            GPT3_MODELS,
            GPT4_MODELS,
            TURBO_MODELS,
        )

        return {
            **GPT4_MODELS,
            **TURBO_MODELS,
            **GPT3_5_MODELS,
            **GPT3_MODELS,
            **AZURE_TURBO_MODELS,
            **CLAUDE_MODELS,
        }
    except ImportError as exc:
        raise Exception(IMPORT_ERROR_MESSAGE) from exc


def chat_models():
    try:
        from llama_index.llms.openai_utils import (
            AZURE_TURBO_MODELS,
            GPT4_MODELS,
            TURBO_MODELS,
        )

        return {
            **GPT4_MODELS,
            **TURBO_MODELS,
            **AZURE_TURBO_MODELS,
        }
    except ImportError as exc:
        raise Exception(IMPORT_ERROR_MESSAGE) from exc


DISCONTINUED_MODELS = {
    "code-davinci-002": 8001,
    "code-davinci-001": 8001,
    "code-cushman-002": 2048,
    "code-cushman-001": 2048,
}

DEFAULT_MODEL = "gpt-3.5-turbo"


CLUADE_MODEL_FULLVERSION_MAP = {
    "claude-instant-1": "claude-instant-1.2",
    "claude-2": "claude-2.0",
}

ALL_AVAILABLE_MODELS = all_available_models()

CHAT_MODELS = chat_models()


def is_chat_model(model: str) -> bool:
    """Check if a given model is a chat-based language model.

    This function takes a model name or identifier as input and determines whether
    the model is designed for chat-based language generation, conversation, or
    interaction.

    Args:
        model (str): The name or identifier of the model to be checked.

    Returns:
        bool: True if the provided model is a chat-based language model,
        False otherwise.
    """
    return model in CHAT_MODELS


def modelname_to_contextsize(modelname: str) -> int:
    """Calculate the maximum number of tokens possible to generate for a model.

    Args:
        modelname: The modelname we want to know the context size for.

    Returns:
        The maximum context size

    Example:
        .. code-block:: python

            max_tokens = modelname_to_contextsize("text-davinci-003")
    """
    # handling finetuned models
    if "ft-" in modelname:  # legacy fine-tuning
        modelname = modelname.split(":")[0]
    elif modelname.startswith("ft:"):
        modelname = modelname.split(":")[1]

    if modelname in DISCONTINUED_MODELS:
        raise ValueError(
            f"Model {modelname} has been discontinued. " "Please choose another model."
        )

    context_size = ALL_AVAILABLE_MODELS.get(modelname, None)

    if context_size is None:
        raise ValueError(
            f"Unknown model: {modelname}. Please provide a valid model name."
            "Known models are: " + ", ".join(ALL_AVAILABLE_MODELS.keys())
        )

    return context_size



from __future__ import annotations
from unbound import Unbound
import logging
from typing import (
    TYPE_CHECKING,
    Any,
    Dict,
    Iterator,
    List,
    Mapping,
    Optional,
    TypedDict,
    Union,
    cast,
)

try:
    from langchain.callbacks.manager import CallbackManagerForLLMRun
    from langchain.chat_models.base import SimpleChatModel
    from langchain.pydantic_v1 import Field, PrivateAttr
    from langchain.schema.messages import (
        AIMessage,
        AIMessageChunk,
        BaseMessage,
        ChatMessage,
        FunctionMessage,
        HumanMessage,
        SystemMessage,
    )
    from langchain.schema.output import (
        ChatGenerationChunk,
    )
except ImportError as exc:
    raise Exception(
        "Langchain is not installed.Please install it with `pip install langchain`."
    ) from exc


logger = logging.getLogger(__name__)

if TYPE_CHECKING:
    pass


IMPORT_ERROR_MESSAGE = (
    "Unbound is not installed.Please install it with `pip install unbound`."
)


def convert_message_to_dict(message: BaseMessage) -> dict:
    message_dict: Dict[str, Any]
    if isinstance(message, ChatMessage):
        message_dict = {"role": message.role, "content": message.content}
    elif isinstance(message, HumanMessage):
        message_dict = {"role": "user", "content": message.content}
    elif isinstance(message, AIMessage):
        message_dict = {"role": "assistant", "content": message.content}
        if "function_call" in message.additional_kwargs:
            message_dict["function_call"] = message.additional_kwargs["function_call"]
            # If function call only, content is None not empty string
            if message_dict["content"] == "":
                message_dict["content"] = None
    elif isinstance(message, SystemMessage):
        message_dict = {"role": "system", "content": message.content}
    elif isinstance(message, FunctionMessage):
        message_dict = {
            "role": "function",
            "content": message.content,
            "name": message.name,
        }
    else:
        raise TypeError(f"Got unknown type {message}")
    if "name" in message.additional_kwargs:
        message_dict["name"] = message.additional_kwargs["name"]
    return message_dict


class Message(TypedDict):
    role: str
    content: str


class ChatUnbound(SimpleChatModel):
    """`Unbound` Chat large language models.
    To use, you should have the ``unbound`` python package installed, and the
    environment variable ``UNBOUND_API_KEY``, set with your API key, or pass
    it as a named parameter to the `Unbound` constructor.
    NOTE: You can install unbound using ``pip install unbound``
    Example:
        .. code-block:: python
            import unbound
            from langchain.chat_models import ChatUnbound
            # Simplest invocation for an openai provider. Can be extended to
            # others as well
            llm_option = unbound.LLMOptions(
                provider="openai",
                # Checkout the docs for the virtual-api-key
                virtual_key="openai-virtual-key",
                model="text-davinci-003"
            )
            # Initialise the client
            client = ChatUnbound(
                api_key="UNBOUND_API_KEY",
                mode="single"
            ).add_llms(llm_params=llm_option)
            response = client("What are the biggest risks facing humanity?")
    """

    model: Optional[str] = Field(default="gpt-3.5-turbo")

    _client: Any = PrivateAttr()
    api_key: Optional[str] = None
    base_url: Optional[str] = None
    virtual_key: Optional[str] = None
    config: Optional[Union[Mapping, str]] = None
    provider: Optional[str] = None
    trace_id: Optional[str] = None
    custom_metadata: Optional[str] = None
    streaming: bool = False

    def __init__(
        self,
        *,
        api_key: Optional[str] = None,
        base_url: Optional[str] = None,
        virtual_key: Optional[str] = None,
        config: Optional[Union[Mapping, str]] = None,
        provider: Optional[str] = None,
        trace_id: Optional[str] = None,
        custom_metadata: Optional[str] = None,
        **kwargs,
    ) -> None:
        super().__init__()

        self._client = Unbound(
            api_key=api_key,
            base_url=base_url,
            virtual_key=virtual_key,
            config=config,
            provider=provider,
            trace_id=trace_id,
            metadata=custom_metadata,
            **kwargs,
        )
        self.model = None

    def _call(
        self,
        messages: List[BaseMessage],
        stop: Optional[List[str]] = None,
        run_manager: Optional[CallbackManagerForLLMRun] = None,
        **kwargs: Any,
    ) -> str:
        """Call Unbound's chatCompletions endpoint.
        Args:
            prompt: The prompt to pass into the model.
            stop: Optional list of stop words to use when generating.
        Returns:
            The string generated by the provider set in the initialisation of the LLM.
        Example:
            .. code-block:: python
                message = [{
                    "role": "user",
                    "content": "Tell me a joke."
                }]
                response = unbound(message)
        """
        _messages = cast(Message, self._create_message_dicts(messages))
        response = self._client.chat.completions.create(
            messages=_messages, stream=False, stop=stop, **kwargs
        )
        message = response.choices[0].message
        return message.get("content", "") if message else ""

    def _create_message_dicts(
        self, messages: List[BaseMessage]
    ) -> List[Dict[str, Any]]:
        message_dicts = [convert_message_to_dict(m) for m in messages]
        return message_dicts

    def _stream(
        self,
        messages: List[BaseMessage],
        stop: Optional[List[str]] = None,
        run_manager: Optional[CallbackManagerForLLMRun] = None,
        **kwargs: Any,
    ) -> Iterator[ChatGenerationChunk]:
        """Call Unbound completion_stream and return the resulting generator.
        Args:
            prompt: The prompt to pass into the model.
            stop: Optional list of stop words to use when generating.
        Returns:
            A generator representing the stream of tokens from Unbound.
        Example:
            .. code-block:: python
                prompt = "Write a poem about a stream."
                generator = unbound.stream(prompt)
                for token in generator:
                    yield token
        """
        _messages = cast(Message, self._create_message_dicts(messages))
        response = self._client.chat.completions.create(
            messages=_messages, stream=True, stop=stop, **kwargs
        )
        for token in response:
            _content = token.choices[0].delta.get("content") or ""
            chunk = ChatGenerationChunk(message=AIMessageChunk(content=_content))
            yield chunk
            if run_manager:
                run_manager.on_llm_new_token(chunk.text, chunk=chunk)

    @property
    def _llm_type(self) -> str:
        """Return type of llm."""
        return "unbound-gateway"



import json
from typing import Dict, Iterable, Optional, Union
import httpx
from .utils import parse_headers
from typing import List, Any
from pydantic import BaseModel, PrivateAttr

__all__ = [
    "ChatCompletions",
    "ChatCompletionMessage",
    "ChatCompletionMessageToolCall",
    "FunctionCall",
    "TopLogprob",
    "ChatCompletionTokenLogprob",
    "ChoiceLogprobs",
    "Choice",
    "Usage",
    "DeltaToolCall",
    "Delta",
    "StreamChoice",
    "ChatCompletionChunk",
    "ChoiceLogprobs",
]


class TopLogprob(BaseModel, extra="allow"):
    token: Optional[str]
    bytes: Optional[List[int]] = None
    logprob: Optional[float]


class ChatCompletionTokenLogprob(BaseModel, extra="allow"):
    token: Optional[str]
    bytes: Optional[List[int]] = None
    logprob: Optional[float]
    top_logprobs: Optional[List[TopLogprob]]


class ChoiceLogprobs(BaseModel, extra="allow"):
    content: Optional[List[ChatCompletionTokenLogprob]] = None
    refusal: Optional[List[ChatCompletionTokenLogprob]] = None


class Usage(BaseModel, extra="allow"):
    prompt_tokens: Optional[int] = None
    completion_tokens: Optional[int] = None
    total_tokens: Optional[int] = None


class DeltaToolCallFunction(BaseModel, extra="allow"):
    arguments: Optional[str] = None
    name: Optional[str] = None


class DeltaToolCall(BaseModel, extra="allow"):
    index: Optional[int]
    id: Optional[str] = None
    function: Optional[DeltaToolCallFunction] = None
    type: Optional[str] = None


class Delta(BaseModel, extra="allow"):
    role: Optional[str] = None
    content: Optional[str] = ""
    tool_calls: Optional[List[DeltaToolCall]] = None
    refusal: Optional[str] = None


class StreamChoice(BaseModel, extra="allow"):
    index: Optional[int] = None
    delta: Optional[Delta]
    finish_reason: Optional[str] = None
    logprobs: Optional[ChoiceLogprobs] = None

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def __getitem__(self, key):
        return getattr(self, key, None)


class FunctionCall(BaseModel, extra="allow"):
    arguments: Optional[str]
    name: Optional[str]


class ChatCompletionMessageToolCall(BaseModel, extra="allow"):
    id: Optional[str]
    function: Optional[FunctionCall]
    type: Optional[str]


class ChatCompletionMessage(BaseModel, extra="allow"):
    content: Optional[Union[str, Iterable[Any]]] = None
    role: Optional[str]
    function_call: Optional[FunctionCall] = None
    tool_calls: Optional[List[ChatCompletionMessageToolCall]] = None


class Choice(BaseModel, extra="allow"):
    finish_reason: Optional[str]
    index: Optional[int]
    logprobs: Optional[ChoiceLogprobs] = None
    message: Optional[ChatCompletionMessage]


class ChatCompletions(BaseModel, extra="allow"):
    id: Optional[str]
    choices: Optional[List[Choice]]
    created: Optional[int]
    model: Optional[str]
    object: Optional[str]
    system_fingerprint: Optional[str] = None
    usage: Optional[Usage] = None
    service_tier: Optional[str] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class ChatCompletionChunk(BaseModel, extra="allow"):
    id: Optional[str] = None
    object: Optional[str] = None
    created: Optional[int] = None
    model: Optional[str] = None
    choices: Optional[List[StreamChoice]]
    service_tier: Optional[str] = None

    def __str__(self):
        return json.dumps(self.dict(), indent=4)

    def __getitem__(self, key):
        return getattr(self, key, None)

    def get(self, key: str, default: Optional[Any] = None):
        return getattr(self, key, None) or default



# File generated from our OpenAPI spec by Stainless. See CONTRIBUTING.md for details.

from typing import List, Union, Optional
from typing_extensions import Literal, TypeAlias

from ...._models import BaseModel
from .message_content import MessageContent
from ..code_interpreter_tool import CodeInterpreterTool

__all__ = [
    "Message",
    "Attachment",
    "AttachmentTool",
    "AttachmentToolAssistantToolsFileSearchTypeOnly",
    "IncompleteDetails",
]


class AttachmentToolAssistantToolsFileSearchTypeOnly(BaseModel):
    type: Literal["file_search"]
    """The type of tool being defined: `file_search`"""


AttachmentTool: TypeAlias = Union[CodeInterpreterTool, AttachmentToolAssistantToolsFileSearchTypeOnly]


class Attachment(BaseModel):
    file_id: Optional[str] = None
    """The ID of the file to attach to the message."""

    tools: Optional[List[AttachmentTool]] = None
    """The tools to add this file to."""


class IncompleteDetails(BaseModel):
    reason: Literal["content_filter", "max_tokens", "run_cancelled", "run_expired", "run_failed"]
    """The reason the message is incomplete."""


class Message(BaseModel):
    id: str
    """The identifier, which can be referenced in API endpoints."""

    assistant_id: Optional[str] = None
    """
    If applicable, the ID of the
    [assistant](https://platform.openai.com/docs/api-reference/assistants) that
    authored this message.
    """

    attachments: Optional[List[Attachment]] = None
    """A list of files attached to the message, and the tools they were added to."""

    completed_at: Optional[int] = None
    """The Unix timestamp (in seconds) for when the message was completed."""

    content: List[MessageContent]
    """The content of the message in array of text and/or images."""

    created_at: int
    """The Unix timestamp (in seconds) for when the message was created."""

    incomplete_at: Optional[int] = None
    """The Unix timestamp (in seconds) for when the message was marked as incomplete."""

    incomplete_details: Optional[IncompleteDetails] = None
    """On an incomplete message, details about why the message is incomplete."""

    metadata: Optional[object] = None
    """Set of 16 key-value pairs that can be attached to an object.

    This can be useful for storing additional information about the object in a
    structured format. Keys can be a maximum of 64 characters long and values can be
    a maxium of 512 characters long.
    """

    object: Literal["thread.message"]
    """The object type, which is always `thread.message`."""

    role: Literal["user", "assistant"]
    """The entity that produced the message. One of `user` or `assistant`."""

    run_id: Optional[str] = None
    """
    The ID of the [run](https://platform.openai.com/docs/api-reference/runs)
    associated with the creation of this message. Value is `null` when messages are
    created manually using the create message or create thread endpoints.
    """

    status: Literal["in_progress", "incomplete", "completed"]
    """
    The status of the message, which can be either `in_progress`, `incomplete`, or
    `completed`.
    """

    thread_id: str
    """
    The [thread](https://platform.openai.com/docs/api-reference/threads) ID that
    this message belongs to.
    """



import json
from typing import Dict, Literal, Optional
import httpx

from unbound._vendor.openai.types.file_object import FileObject
from .utils import parse_headers
from pydantic import BaseModel, PrivateAttr


class Upload(BaseModel, extra="allow"):
    id: Optional[str]
    bytes: Optional[int]
    created_at: Optional[int]
    expires_at: Optional[int]
    filename: Optional[str]
    object: Optional[Literal["upload"]]
    purpose: Optional[str]
    status: Optional[Literal["pending", "completed", "cancelled", "expired"]]
    file: Optional[FileObject] = None
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)


class UploadPart(BaseModel, extra="allow"):
    id: Optional[str]
    created_at: Optional[int]
    object: Optional[Literal["upload.part"]]
    upload_id: Optional[str]
    _headers: Optional[httpx.Headers] = PrivateAttr()

    def __str__(self):
        del self._headers
        return json.dumps(self.dict(), indent=4)

    def get_headers(self) -> Optional[Dict[str, str]]:
        return parse_headers(self._headers)
